from typing import Any, Optional, Sequence, Union
from dataclasses import dataclass

from . import RequestMsg, ReplyMsg, Message, SimpleMessage
from hedgehog.protocol.proto import speaker_pb2
from hedgehog.utils import protobuf

__all__ = ['Action']

# <default GSL customizable: module-header />


@RequestMsg.message(speaker_pb2.SpeakerAction, 'speaker_action', fields=('frequency',))
@dataclass(frozen=True, repr=False)
class Action(SimpleMessage):
    frequency: Optional[int]

    def __post_init__(self):
        # <default GSL customizable: Action-init-validation>
        pass
        # </GSL customizable: Action-init-validation>

    # <default GSL customizable: Action-extra-members />

    @classmethod
    def _parse(cls, msg: speaker_pb2.SpeakerAction) -> 'Action':
        frequency = msg.frequency
        return cls(frequency if frequency != 0 else None)

    def _serialize(self, msg: speaker_pb2.SpeakerAction) -> None:
        msg.frequency = self.frequency if self.frequency is not None else 0



from typing import Any, Sequence, Union
from dataclasses import dataclass

from . import RequestMsg, ReplyMsg, Message, SimpleMessage
from hedgehog.protocol.proto import io_pb2
from hedgehog.utils import protobuf

__all__ = ['Action', 'CommandRequest', 'CommandReply', 'CommandSubscribe', 'CommandUpdate']

# <GSL customizable: module-header>
from hedgehog.protocol.errors import InvalidCommandError
from hedgehog.protocol.proto.io_pb2 import INPUT_FLOATING, INPUT_PULLUP, INPUT_PULLDOWN
from hedgehog.protocol.proto.io_pb2 import OUTPUT_OFF, OUTPUT_ON
from hedgehog.protocol.proto.io_pb2 import OUTPUT, PULLUP, PULLDOWN, LEVEL
from hedgehog.protocol.proto.subscription_pb2 import Subscription

__all__ += [
    'INPUT_FLOATING', 'INPUT_PULLUP', 'INPUT_PULLDOWN',
    'OUTPUT_OFF', 'OUTPUT_ON',
    'OUTPUT', 'PULLUP', 'PULLDOWN', 'LEVEL',
    'Subscription',
]


def _check_flags(flags: int) -> None:
    if flags & OUTPUT and flags & (PULLUP | PULLDOWN):
        raise InvalidCommandError("only input ports can be set to pullup or pulldown")
    if not flags & OUTPUT and flags & LEVEL:
        raise InvalidCommandError("only output ports can be set to on")
    if flags & PULLUP and flags & PULLDOWN:
        raise InvalidCommandError("pullup and pulldown are mutually exclusive")
# </GSL customizable: module-header>


@RequestMsg.message(io_pb2.IOAction, 'io_action', fields=('port', 'flags',))
@dataclass(frozen=True, repr=False)
class Action(SimpleMessage):
    port: int
    flags: int

    def __post_init__(self):
        # <GSL customizable: Action-init-validation>
        _check_flags(self.flags)
        # </GSL customizable: Action-init-validation>

    # <GSL customizable: Action-extra-members>
    @property
    def output(self) -> bool:
        return (self.flags & OUTPUT) != 0

    @property
    def pullup(self) -> bool:
        return (self.flags & PULLUP) != 0

    @property
    def pulldown(self) -> bool:
        return (self.flags & PULLDOWN) != 0

    @property
    def level(self) -> bool:
        return (self.flags & LEVEL) != 0
    # </GSL customizable: Action-extra-members>

    @classmethod
    def _parse(cls, msg: io_pb2.IOAction) -> 'Action':
        port = msg.port
        flags = msg.flags
        return cls(port, flags)

    def _serialize(self, msg: io_pb2.IOAction) -> None:
        msg.port = self.port
        msg.flags = self.flags


@protobuf.message(io_pb2.IOCommandMessage, 'io_command_message', fields=('port',))
@dataclass(frozen=True, repr=False)
class CommandRequest(Message):
    port: int

    def __post_init__(self):
        # <default GSL customizable: CommandRequest-init-validation>
        pass
        # </GSL customizable: CommandRequest-init-validation>

    # <default GSL customizable: CommandRequest-extra-members />

    def _serialize(self, msg: io_pb2.IOCommandMessage) -> None:
        msg.port = self.port


@protobuf.message(io_pb2.IOCommandMessage, 'io_command_message', fields=('port', 'flags',))
@dataclass(frozen=True, repr=False)
class CommandReply(Message):
    port: int
    flags: int

    def __post_init__(self):
        # <GSL customizable: CommandReply-init-validation>
        _check_flags(self.flags)
        # </GSL customizable: CommandReply-init-validation>

    # <GSL customizable: CommandReply-extra-members>
    @property
    def output(self) -> bool:
        return (self.flags & OUTPUT) != 0

    @property
    def pullup(self) -> bool:
        return (self.flags & PULLUP) != 0

    @property
    def pulldown(self) -> bool:
        return (self.flags & PULLDOWN) != 0

    @property
    def level(self) -> bool:
        return (self.flags & LEVEL) != 0
    # </GSL customizable: CommandReply-extra-members>

    def _serialize(self, msg: io_pb2.IOCommandMessage) -> None:
        msg.port = self.port
        msg.flags = self.flags


@protobuf.message(io_pb2.IOCommandMessage, 'io_command_message', fields=('port', 'subscription',))
@dataclass(frozen=True, repr=False)
class CommandSubscribe(Message):
    port: int
    subscription: Subscription

    def __post_init__(self):
        # <default GSL customizable: CommandSubscribe-init-validation>
        pass
        # </GSL customizable: CommandSubscribe-init-validation>

    # <default GSL customizable: CommandSubscribe-extra-members />

    def _serialize(self, msg: io_pb2.IOCommandMessage) -> None:
        msg.port = self.port
        msg.subscription.CopyFrom(self.subscription)


@protobuf.message(io_pb2.IOCommandMessage, 'io_command_message', fields=('port', 'flags', 'subscription',))
@dataclass(frozen=True, repr=False)
class CommandUpdate(Message):
    is_async = True

    port: int
    flags: int
    subscription: Subscription

    def __post_init__(self):
        # <GSL customizable: CommandUpdate-init-validation>
        _check_flags(self.flags)
        # </GSL customizable: CommandUpdate-init-validation>

    # <GSL customizable: CommandUpdate-extra-members>
    @property
    def output(self) -> bool:
        return (self.flags & OUTPUT) != 0

    @property
    def pullup(self) -> bool:
        return (self.flags & PULLUP) != 0

    @property
    def pulldown(self) -> bool:
        return (self.flags & PULLDOWN) != 0

    @property
    def level(self) -> bool:
        return (self.flags & LEVEL) != 0
    # </GSL customizable: CommandUpdate-extra-members>

    def _serialize(self, msg: io_pb2.IOCommandMessage) -> None:
        msg.port = self.port
        msg.flags = self.flags
        msg.subscription.CopyFrom(self.subscription)


@RequestMsg.parser('io_command_message')
def _parse_io_command_message_request(msg: io_pb2.IOCommandMessage) -> Union[CommandRequest, CommandSubscribe]:
    port = msg.port
    flags = msg.flags
    subscription = msg.subscription if msg.HasField('subscription') else None
    # <GSL customizable: _parse_io_command_message_request-return>
    if subscription is None:
        return CommandRequest(port)
    else:
        return CommandSubscribe(port, subscription)
    # </GSL customizable: _parse_io_command_message_request-return>


@ReplyMsg.parser('io_command_message')
def _parse_io_command_message_reply(msg: io_pb2.IOCommandMessage) -> Union[CommandReply, CommandUpdate]:
    port = msg.port
    flags = msg.flags
    subscription = msg.subscription if msg.HasField('subscription') else None
    # <GSL customizable: _parse_io_command_message_reply-return>
    if subscription is None:
        return CommandReply(port, flags)
    else:
        return CommandUpdate(port, flags, subscription)
    # </GSL customizable: _parse_io_command_message_reply-return>



from typing import Any, Sequence, Union
from dataclasses import dataclass

from . import RequestMsg, ReplyMsg, Message, SimpleMessage
from hedgehog.protocol.proto import version_pb2
from hedgehog.utils import protobuf

__all__ = ['Request', 'Reply']

# <default GSL customizable: module-header />


@RequestMsg.message(version_pb2.VersionMessage, 'version_message', fields=())
@dataclass(frozen=True, repr=False)
class Request(SimpleMessage):

    def __post_init__(self):
        # <default GSL customizable: Request-init-validation>
        pass
        # </GSL customizable: Request-init-validation>

    # <default GSL customizable: Request-extra-members />

    @classmethod
    def _parse(cls, msg: version_pb2.VersionMessage) -> 'Request':
        return cls()

    def _serialize(self, msg: version_pb2.VersionMessage) -> None:
        msg.SetInParent()


@ReplyMsg.message(version_pb2.VersionMessage, 'version_message', fields=('uc_id', 'hardware_version', 'firmware_version', 'server_version',))
@dataclass(frozen=True, repr=False)
class Reply(SimpleMessage):
    uc_id: bytes
    hardware_version: str
    firmware_version: str
    server_version: str

    def __post_init__(self):
        # <default GSL customizable: Reply-init-validation>
        pass
        # </GSL customizable: Reply-init-validation>

    # <default GSL customizable: Reply-extra-members />

    @classmethod
    def _parse(cls, msg: version_pb2.VersionMessage) -> 'Reply':
        uc_id = msg.uc_id
        hardware_version = msg.hardware_version
        firmware_version = msg.firmware_version
        server_version = msg.server_version
        return cls(uc_id, hardware_version, firmware_version, server_version)

    def _serialize(self, msg: version_pb2.VersionMessage) -> None:
        msg.uc_id = self.uc_id
        msg.hardware_version = self.hardware_version
        msg.firmware_version = self.firmware_version
        msg.server_version = self.server_version



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: hedgehog/protocol/proto/speaker.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='hedgehog/protocol/proto/speaker.proto',
  package='hedgehog.protocol.proto',
  syntax='proto3',
  serialized_pb=_b('\n%hedgehog/protocol/proto/speaker.proto\x12\x17hedgehog.protocol.proto\"\"\n\rSpeakerAction\x12\x11\n\tfrequency\x18\x01 \x01(\rB+\n\x1f\x61t.pria.hedgehog.protocol.protoB\x08SpeakerPb\x06proto3')
)




_SPEAKERACTION = _descriptor.Descriptor(
  name='SpeakerAction',
  full_name='hedgehog.protocol.proto.SpeakerAction',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='frequency', full_name='hedgehog.protocol.proto.SpeakerAction.frequency', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=66,
  serialized_end=100,
)

DESCRIPTOR.message_types_by_name['SpeakerAction'] = _SPEAKERACTION
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

SpeakerAction = _reflection.GeneratedProtocolMessageType('SpeakerAction', (_message.Message,), dict(
  DESCRIPTOR = _SPEAKERACTION,
  __module__ = 'hedgehog.protocol.proto.speaker_pb2'
  # @@protoc_insertion_point(class_scope:hedgehog.protocol.proto.SpeakerAction)
  ))
_sym_db.RegisterMessage(SpeakerAction)


DESCRIPTOR.has_options = True
DESCRIPTOR._options = _descriptor._ParseOptions(descriptor_pb2.FileOptions(), _b('\n\037at.pria.hedgehog.protocol.protoB\010SpeakerP'))
# @@protoc_insertion_point(module_scope)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: hedgehog/protocol/proto/imu.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from hedgehog.protocol.proto import subscription_pb2 as hedgehog_dot_protocol_dot_proto_dot_subscription__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='hedgehog/protocol/proto/imu.proto',
  package='hedgehog.protocol.proto',
  syntax='proto3',
  serialized_pb=_b('\n!hedgehog/protocol/proto/imu.proto\x12\x17hedgehog.protocol.proto\x1a*hedgehog/protocol/proto/subscription.proto\"\x9a\x01\n\nImuMessage\x12.\n\x04kind\x18\x01 \x01(\x0e\x32 .hedgehog.protocol.proto.ImuKind\x12\t\n\x01x\x18\x02 \x01(\x11\x12\t\n\x01y\x18\x03 \x01(\x11\x12\t\n\x01z\x18\x04 \x01(\x11\x12;\n\x0csubscription\x18\x05 \x01(\x0b\x32%.hedgehog.protocol.proto.Subscription*/\n\x07ImuKind\x12\x08\n\x04RATE\x10\x00\x12\x10\n\x0c\x41\x43\x43\x45LERATION\x10\x01\x12\x08\n\x04POSE\x10\x02\x42\'\n\x1f\x61t.pria.hedgehog.protocol.protoB\x04ImuPb\x06proto3')
  ,
  dependencies=[hedgehog_dot_protocol_dot_proto_dot_subscription__pb2.DESCRIPTOR,])

_IMUKIND = _descriptor.EnumDescriptor(
  name='ImuKind',
  full_name='hedgehog.protocol.proto.ImuKind',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='RATE', index=0, number=0,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='ACCELERATION', index=1, number=1,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='POSE', index=2, number=2,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=263,
  serialized_end=310,
)
_sym_db.RegisterEnumDescriptor(_IMUKIND)

ImuKind = enum_type_wrapper.EnumTypeWrapper(_IMUKIND)
RATE = 0
ACCELERATION = 1
POSE = 2



_IMUMESSAGE = _descriptor.Descriptor(
  name='ImuMessage',
  full_name='hedgehog.protocol.proto.ImuMessage',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='kind', full_name='hedgehog.protocol.proto.ImuMessage.kind', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='x', full_name='hedgehog.protocol.proto.ImuMessage.x', index=1,
      number=2, type=17, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='y', full_name='hedgehog.protocol.proto.ImuMessage.y', index=2,
      number=3, type=17, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='z', full_name='hedgehog.protocol.proto.ImuMessage.z', index=3,
      number=4, type=17, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='subscription', full_name='hedgehog.protocol.proto.ImuMessage.subscription', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=107,
  serialized_end=261,
)

_IMUMESSAGE.fields_by_name['kind'].enum_type = _IMUKIND
_IMUMESSAGE.fields_by_name['subscription'].message_type = hedgehog_dot_protocol_dot_proto_dot_subscription__pb2._SUBSCRIPTION
DESCRIPTOR.message_types_by_name['ImuMessage'] = _IMUMESSAGE
DESCRIPTOR.enum_types_by_name['ImuKind'] = _IMUKIND
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

ImuMessage = _reflection.GeneratedProtocolMessageType('ImuMessage', (_message.Message,), dict(
  DESCRIPTOR = _IMUMESSAGE,
  __module__ = 'hedgehog.protocol.proto.imu_pb2'
  # @@protoc_insertion_point(class_scope:hedgehog.protocol.proto.ImuMessage)
  ))
_sym_db.RegisterMessage(ImuMessage)


DESCRIPTOR.has_options = True
DESCRIPTOR._options = _descriptor._ParseOptions(descriptor_pb2.FileOptions(), _b('\n\037at.pria.hedgehog.protocol.protoB\004ImuP'))
# @@protoc_insertion_point(module_scope)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: hedgehog/protocol/proto/version.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='hedgehog/protocol/proto/version.proto',
  package='hedgehog.protocol.proto',
  syntax='proto3',
  serialized_pb=_b('\n%hedgehog/protocol/proto/version.proto\x12\x17hedgehog.protocol.proto\"k\n\x0eVersionMessage\x12\r\n\x05uc_id\x18\x01 \x01(\x0c\x12\x18\n\x10hardware_version\x18\x02 \x01(\t\x12\x18\n\x10\x66irmware_version\x18\x03 \x01(\t\x12\x16\n\x0eserver_version\x18\x04 \x01(\tB+\n\x1f\x61t.pria.hedgehog.protocol.protoB\x08VersionPb\x06proto3')
)




_VERSIONMESSAGE = _descriptor.Descriptor(
  name='VersionMessage',
  full_name='hedgehog.protocol.proto.VersionMessage',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='uc_id', full_name='hedgehog.protocol.proto.VersionMessage.uc_id', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='hardware_version', full_name='hedgehog.protocol.proto.VersionMessage.hardware_version', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='firmware_version', full_name='hedgehog.protocol.proto.VersionMessage.firmware_version', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='server_version', full_name='hedgehog.protocol.proto.VersionMessage.server_version', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=66,
  serialized_end=173,
)

DESCRIPTOR.message_types_by_name['VersionMessage'] = _VERSIONMESSAGE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

VersionMessage = _reflection.GeneratedProtocolMessageType('VersionMessage', (_message.Message,), dict(
  DESCRIPTOR = _VERSIONMESSAGE,
  __module__ = 'hedgehog.protocol.proto.version_pb2'
  # @@protoc_insertion_point(class_scope:hedgehog.protocol.proto.VersionMessage)
  ))
_sym_db.RegisterMessage(VersionMessage)


DESCRIPTOR.has_options = True
DESCRIPTOR._options = _descriptor._ParseOptions(descriptor_pb2.FileOptions(), _b('\n\037at.pria.hedgehog.protocol.protoB\010VersionP'))
# @@protoc_insertion_point(module_scope)



from typing import Any, Dict, List, Optional, Set, Tuple, Union
from dataclasses import dataclass

from . import RequestMsg, ReplyMsg, Message, SimpleMessage
from hedgehog.protocol.proto import vision_pb2
from hedgehog.utils import protobuf

__all__ = ['OpenCameraAction', 'CloseCameraAction', 'CreateChannelAction', 'UpdateChannelAction', 'DeleteChannelAction', 'ChannelRequest', 'ChannelReply', 'CaptureFrameAction', 'FrameRequest', 'FrameReply']

# <GSL customizable: module-header>
from hedgehog.protocol.proto.vision_pb2 import ChannelOperation
from hedgehog.protocol.proto.vision_pb2 import CREATE, READ, UPDATE, DELETE


@dataclass
class FacesChannel:
    @classmethod
    def _parse(cls, msg: vision_pb2.Channel) -> Tuple[str, 'FacesChannel']:
        return msg.key, cls()

    def _serialize(self, msg: vision_pb2.Channel, key: str) -> None:
        msg.key = key
        msg.faces.SetInParent()


@dataclass
class BlobsChannel:
    hsv_min: Tuple[int, int, int]
    hsv_max: Tuple[int, int, int]

    @staticmethod
    def _pack(hsv: Tuple[int, int, int]) -> int:
        return int.from_bytes(hsv, 'big')

    @staticmethod
    def _unpack(hsv: int) -> Tuple[int, int, int]:
        return tuple(hsv.to_bytes(3, 'big'))

    @classmethod
    def _parse(cls, msg: vision_pb2.Channel) -> Tuple[str, 'BlobsChannel']:
        hsv_min = BlobsChannel._unpack(msg.blobs.hsv_min)
        hsv_max = BlobsChannel._unpack(msg.blobs.hsv_max)
        return msg.key, cls(hsv_min, hsv_max)

    def _serialize(self, msg: vision_pb2.Channel, key: str) -> None:
        msg.key = key
        msg.blobs.hsv_min = BlobsChannel._pack(self.hsv_min)
        msg.blobs.hsv_max = BlobsChannel._pack(self.hsv_max)


Channel = Union[FacesChannel, BlobsChannel]


def _parse_channel(msg: vision_pb2.Channel) -> Tuple[str, Channel]:
    if msg.HasField('faces'):
        return FacesChannel._parse(msg)
    elif msg.HasField('blobs'):
        return BlobsChannel._parse(msg)
    else:  # pragma: nocover
        assert False


@dataclass
class Face:
    bounding_rect: Tuple[int, int, int, int]

    @classmethod
    def _parse(cls, msg: vision_pb2.Face) -> 'Face':
        return cls(
            (msg.x, msg.y, msg.width, msg.height),
        )

    def _serialize(self, msg: vision_pb2.Face) -> None:
        msg.x, msg.y, msg.width, msg.height = self.bounding_rect


@dataclass
class FacesFeature:
    faces: List[Face]

    @classmethod
    def _parse(cls, msg: vision_pb2.Feature) -> 'FacesFeature':
        return cls([Face._parse(face) for face in msg.faces.faces])

    def _serialize(self, msg: vision_pb2.Feature) -> None:
        msg.faces.SetInParent()
        for face in self.faces:
            face._serialize(msg.faces.faces.add())


@dataclass
class Blob:
    bounding_rect: Tuple[int, int, int, int]
    centroid: Tuple[int, int]
    confidence: float

    @classmethod
    def _parse(cls, msg: vision_pb2.Blob) -> 'Blob':
        return cls(
            (msg.x, msg.y, msg.width, msg.height),
            (msg.cx, msg.cy),
            msg.confidence,
        )

    def _serialize(self, msg: vision_pb2.Face) -> None:
        msg.x, msg.y, msg.width, msg.height = self.bounding_rect
        msg.cx, msg.cy = self.centroid
        msg.confidence = self.confidence


@dataclass
class BlobsFeature:
    blobs: List[Blob]

    @classmethod
    def _parse(cls, msg: vision_pb2.Feature) -> 'BlobsFeature':
        return cls([Blob._parse(blob) for blob in msg.blobs.blobs])

    def _serialize(self, msg: vision_pb2.Feature) -> None:
        msg.blobs.SetInParent()
        for blob in self.blobs:
            blob._serialize(msg.blobs.blobs.add())


Feature = Union[FacesFeature, BlobsFeature]


def _parse_feature(msg: vision_pb2.Feature) -> Feature:
    if msg.HasField('faces'):
        return FacesFeature._parse(msg)
    elif msg.HasField('blobs'):
        return BlobsFeature._parse(msg)
    else:  # pragma: nocover
        assert False


__all__ += [
    'ChannelOperation',
    'CREATE', 'READ', 'UPDATE', 'DELETE',
    'FacesChannel', 'BlobsChannel', 'Channel',
]
# </GSL customizable: module-header>


@protobuf.message(vision_pb2.VisionCameraAction, 'vision_camera_action', fields=())
@dataclass(frozen=True, repr=False)
class OpenCameraAction(Message):

    def __post_init__(self):
        # <default GSL customizable: OpenCameraAction-init-validation>
        pass
        # </GSL customizable: OpenCameraAction-init-validation>

    # <default GSL customizable: OpenCameraAction-extra-members />

    def _serialize(self, msg: vision_pb2.VisionCameraAction) -> None:
        msg.open = True


@protobuf.message(vision_pb2.VisionCameraAction, 'vision_camera_action', fields=())
@dataclass(frozen=True, repr=False)
class CloseCameraAction(Message):

    def __post_init__(self):
        # <default GSL customizable: CloseCameraAction-init-validation>
        pass
        # </GSL customizable: CloseCameraAction-init-validation>

    # <default GSL customizable: CloseCameraAction-extra-members />

    def _serialize(self, msg: vision_pb2.VisionCameraAction) -> None:
        msg.open = False


@protobuf.message(vision_pb2.VisionChannelMessage, 'vision_channel_message', fields=('channels',))
@dataclass(frozen=True, repr=False)
class CreateChannelAction(Message):
    channels: Dict[str, Channel]

    def __post_init__(self):
        # <default GSL customizable: CreateChannelAction-init-validation>
        pass
        # </GSL customizable: CreateChannelAction-init-validation>

    # <default GSL customizable: CreateChannelAction-extra-members />

    def _serialize(self, msg: vision_pb2.VisionChannelMessage) -> None:
        # <GSL customizable: CreateChannelAction-serialize-channels>
        msg.op = CREATE
        for key, channel in self.channels.items():
            channel._serialize(msg.channels.add(), key)
        # </GSL customizable: CreateChannelAction-serialize-channels>


@protobuf.message(vision_pb2.VisionChannelMessage, 'vision_channel_message', fields=('channels',))
@dataclass(frozen=True, repr=False)
class UpdateChannelAction(Message):
    channels: Dict[str, Channel]

    def __post_init__(self):
        # <default GSL customizable: UpdateChannelAction-init-validation>
        pass
        # </GSL customizable: UpdateChannelAction-init-validation>

    # <default GSL customizable: UpdateChannelAction-extra-members />

    def _serialize(self, msg: vision_pb2.VisionChannelMessage) -> None:
        # <GSL customizable: UpdateChannelAction-serialize-channels>
        msg.op = UPDATE
        for key, channel in self.channels.items():
            channel._serialize(msg.channels.add(), key)
        # </GSL customizable: UpdateChannelAction-serialize-channels>


@protobuf.message(vision_pb2.VisionChannelMessage, 'vision_channel_message', fields=('keys',))
@dataclass(frozen=True, repr=False)
class DeleteChannelAction(Message):
    keys: Set[str]

    def __post_init__(self):
        # <default GSL customizable: DeleteChannelAction-init-validation>
        pass
        # </GSL customizable: DeleteChannelAction-init-validation>

    # <default GSL customizable: DeleteChannelAction-extra-members />

    def _serialize(self, msg: vision_pb2.VisionChannelMessage) -> None:
        # <GSL customizable: DeleteChannelAction-serialize-keys>
        msg.op = DELETE
        for key in self.keys:
            msg.channels.add().key = key
        # </GSL customizable: DeleteChannelAction-serialize-keys>


@protobuf.message(vision_pb2.VisionChannelMessage, 'vision_channel_message', fields=('keys',))
@dataclass(frozen=True, repr=False)
class ChannelRequest(Message):
    keys: Set[str]

    def __post_init__(self):
        # <default GSL customizable: ChannelRequest-init-validation>
        pass
        # </GSL customizable: ChannelRequest-init-validation>

    # <default GSL customizable: ChannelRequest-extra-members />

    def _serialize(self, msg: vision_pb2.VisionChannelMessage) -> None:
        # <GSL customizable: ChannelRequest-serialize-keys>
        msg.op = READ
        for key in self.keys:
            msg.channels.add().key = key
        # </GSL customizable: ChannelRequest-serialize-keys>


@ReplyMsg.message(vision_pb2.VisionChannelMessage, 'vision_channel_message', fields=('channels',))
@dataclass(frozen=True, repr=False)
class ChannelReply(SimpleMessage):
    channels: Dict[str, Channel]

    def __post_init__(self):
        # <default GSL customizable: ChannelReply-init-validation>
        pass
        # </GSL customizable: ChannelReply-init-validation>

    # <default GSL customizable: ChannelReply-extra-members />

    @classmethod
    def _parse(cls, msg: vision_pb2.VisionChannelMessage) -> 'ChannelReply':
        # <GSL customizable: ChannelReply-parse-channels>
        channels = {key: channel for key, channel in (_parse_channel(msg) for msg in msg.channels)}
        # </GSL customizable: ChannelReply-parse-channels>
        return cls(channels)

    def _serialize(self, msg: vision_pb2.VisionChannelMessage) -> None:
        # <GSL customizable: ChannelReply-serialize-channels>
        msg.op = READ
        for key, channel in self.channels.items():
            channel._serialize(msg.channels.add(), key)
        # </GSL customizable: ChannelReply-serialize-channels>


@RequestMsg.message(vision_pb2.VisionCaptureFrameAction, 'vision_capture_frame_action', fields=())
@dataclass(frozen=True, repr=False)
class CaptureFrameAction(SimpleMessage):

    def __post_init__(self):
        # <default GSL customizable: CaptureFrameAction-init-validation>
        pass
        # </GSL customizable: CaptureFrameAction-init-validation>

    # <default GSL customizable: CaptureFrameAction-extra-members />

    @classmethod
    def _parse(cls, msg: vision_pb2.VisionCaptureFrameAction) -> 'CaptureFrameAction':
        return cls()

    def _serialize(self, msg: vision_pb2.VisionCaptureFrameAction) -> None:
        msg.SetInParent()


@RequestMsg.message(vision_pb2.VisionFrameMessage, 'vision_frame_message', fields=('highlight',))
@dataclass(frozen=True, repr=False)
class FrameRequest(SimpleMessage):
    highlight: Optional[str]

    def __post_init__(self):
        # <default GSL customizable: FrameRequest-init-validation>
        pass
        # </GSL customizable: FrameRequest-init-validation>

    # <default GSL customizable: FrameRequest-extra-members />

    @classmethod
    def _parse(cls, msg: vision_pb2.VisionFrameMessage) -> 'FrameRequest':
        highlight = msg.highlight
        return cls(highlight if highlight != '' else None)

    def _serialize(self, msg: vision_pb2.VisionFrameMessage) -> None:
        msg.highlight = self.highlight if self.highlight is not None else ''


@ReplyMsg.message(vision_pb2.VisionFrameMessage, 'vision_frame_message', fields=('highlight', 'frame',))
@dataclass(frozen=True, repr=False)
class FrameReply(SimpleMessage):
    highlight: Optional[str]
    frame: bytes

    def __post_init__(self):
        # <default GSL customizable: FrameReply-init-validation>
        pass
        # </GSL customizable: FrameReply-init-validation>

    # <default GSL customizable: FrameReply-extra-members />

    @classmethod
    def _parse(cls, msg: vision_pb2.VisionFrameMessage) -> 'FrameReply':
        highlight = msg.highlight
        frame = msg.frame
        return cls(highlight if highlight != '' else None, frame)

    def _serialize(self, msg: vision_pb2.VisionFrameMessage) -> None:
        msg.highlight = self.highlight if self.highlight is not None else ''
        msg.frame = self.frame


@RequestMsg.message(vision_pb2.VisionFeatureMessage, 'vision_feature_message', fields=('channel',))
@dataclass(frozen=True, repr=False)
class FeatureRequest(SimpleMessage):
    channel: str

    def __post_init__(self):
        # <default GSL customizable: FeatureRequest-init-validation>
        pass
        # </GSL customizable: FeatureRequest-init-validation>

    # <default GSL customizable: FeatureRequest-extra-members />

    @classmethod
    def _parse(cls, msg: vision_pb2.VisionFeatureMessage) -> 'FeatureRequest':
        channel = msg.channel
        return cls(channel)

    def _serialize(self, msg: vision_pb2.VisionFeatureMessage) -> None:
        msg.channel = self.channel


@ReplyMsg.message(vision_pb2.VisionFeatureMessage, 'vision_feature_message', fields=('channel', 'feature',))
@dataclass(frozen=True, repr=False)
class FeatureReply(SimpleMessage):
    channel: str
    feature: Feature

    def __post_init__(self):
        # <default GSL customizable: FeatureReply-init-validation>
        pass
        # </GSL customizable: FeatureReply-init-validation>

    # <default GSL customizable: FeatureReply-extra-members />

    @classmethod
    def _parse(cls, msg: vision_pb2.VisionFeatureMessage) -> 'FeatureReply':
        channel = msg.channel
        # <GSL customizable: FeatureReply-parse-feature>
        feature = _parse_feature(msg.feature)
        # </GSL customizable: FeatureReply-parse-feature>
        return cls(channel, feature)

    def _serialize(self, msg: vision_pb2.VisionFeatureMessage) -> None:
        msg.channel = self.channel
        # <GSL customizable: FeatureReply-serialize-feature>
        self.feature._serialize(msg.feature)
        # </GSL customizable: FeatureReply-serialize-feature>


@RequestMsg.parser('vision_camera_action')
def _parse_vision_camera_action_request(msg: vision_pb2.VisionCameraAction) -> Union[OpenCameraAction, CloseCameraAction]:
    open = msg.open
    # <GSL customizable: _parse_vision_camera_action_request-return>
    if open:
        return OpenCameraAction()
    else:
        return CloseCameraAction()
    # </GSL customizable: _parse_vision_camera_action_request-return>


@RequestMsg.parser('vision_channel_message')
def _parse_vision_channel_message_request(msg: vision_pb2.VisionChannelMessage) -> Union[CreateChannelAction, UpdateChannelAction, DeleteChannelAction, ChannelRequest]:
    op = msg.op
    channels = msg.channels
    # <GSL customizable: _parse_vision_channel_message_request-return>
    if op in {READ, DELETE}:
        keys = {msg.key for msg in channels}
        if op == READ:
            return ChannelRequest(keys)
        else:
            return DeleteChannelAction(keys)
    elif op in {CREATE, UPDATE}:
        channels = {key: channel for key, channel in (_parse_channel(msg) for msg in msg.channels)}
        if op == CREATE:
            return CreateChannelAction(channels)
        else:
            return UpdateChannelAction(channels)
    else:  # pragma: nocover
        assert False
    # </GSL customizable: _parse_vision_channel_message_request-return>



from typing import Any, Sequence, Union
from dataclasses import dataclass

from . import RequestMsg, ReplyMsg, Message, SimpleMessage
from hedgehog.protocol.proto import process_pb2
from hedgehog.utils import protobuf

__all__ = ['ExecuteAction', 'ExecuteReply', 'StreamAction', 'StreamUpdate', 'SignalAction', 'ExitUpdate']

# <GSL customizable: module-header>
from hedgehog.protocol.errors import InvalidCommandError
from hedgehog.protocol.proto.process_pb2 import STDIN, STDOUT, STDERR

__all__ += ['STDIN', 'STDOUT', 'STDERR']
# </GSL customizable: module-header>


@RequestMsg.message(process_pb2.ProcessExecuteAction, 'process_execute_action', fields=('args', 'working_dir',))
@dataclass(frozen=True, repr=False)
class ExecuteAction(SimpleMessage):
    args: Sequence[str]
    working_dir: str = None

    def __init__(self, *args: str, working_dir: str=None) -> None:
        object.__setattr__(self, 'args', args)
        object.__setattr__(self, 'working_dir', working_dir)
        self.__post_init__()

    def __post_init__(self):
        # <default GSL customizable: ExecuteAction-init-validation>
        pass
        # </GSL customizable: ExecuteAction-init-validation>

    # <default GSL customizable: ExecuteAction-extra-members />

    @classmethod
    def _parse(cls, msg: process_pb2.ProcessExecuteAction) -> 'ExecuteAction':
        args = msg.args
        working_dir = msg.working_dir if msg.working_dir != '' else None
        return cls(*args, working_dir=working_dir)

    def _serialize(self, msg: process_pb2.ProcessExecuteAction) -> None:
        msg.args.extend(self.args)
        if self.working_dir is not None:
            msg.working_dir = self.working_dir


@ReplyMsg.message(process_pb2.ProcessExecuteReply, 'process_execute_reply', fields=('pid',))
@dataclass(frozen=True, repr=False)
class ExecuteReply(SimpleMessage):
    pid: int

    def __post_init__(self):
        # <default GSL customizable: ExecuteReply-init-validation>
        pass
        # </GSL customizable: ExecuteReply-init-validation>

    # <default GSL customizable: ExecuteReply-extra-members />

    @classmethod
    def _parse(cls, msg: process_pb2.ProcessExecuteReply) -> 'ExecuteReply':
        pid = msg.pid
        return cls(pid)

    def _serialize(self, msg: process_pb2.ProcessExecuteReply) -> None:
        msg.pid = self.pid


@RequestMsg.message(process_pb2.ProcessStreamMessage, 'process_stream_message', fields=('pid', 'fileno', 'chunk',))
@dataclass(frozen=True, repr=False)
class StreamAction(SimpleMessage):
    pid: int
    fileno: int
    chunk: bytes = b''

    def __post_init__(self):
        # <GSL customizable: StreamAction-init-validation>
        if self.fileno != STDIN:
            raise InvalidCommandError("only STDIN is writable")
        # </GSL customizable: StreamAction-init-validation>

    # <default GSL customizable: StreamAction-extra-members />

    @classmethod
    def _parse(cls, msg: process_pb2.ProcessStreamMessage) -> 'StreamAction':
        pid = msg.pid
        fileno = msg.fileno
        chunk = msg.chunk
        return cls(pid, fileno, chunk)

    def _serialize(self, msg: process_pb2.ProcessStreamMessage) -> None:
        msg.pid = self.pid
        msg.fileno = self.fileno
        msg.chunk = self.chunk


@ReplyMsg.message(process_pb2.ProcessStreamMessage, 'process_stream_message', fields=('pid', 'fileno', 'chunk',))
@dataclass(frozen=True, repr=False)
class StreamUpdate(SimpleMessage):
    is_async = True

    pid: int
    fileno: int
    chunk: bytes = b''

    def __post_init__(self):
        # <GSL customizable: StreamUpdate-init-validation>
        if self.fileno not in (STDOUT, STDERR):
            raise InvalidCommandError("only STDOUT and STDERR are readable")
        # </GSL customizable: StreamUpdate-init-validation>

    # <default GSL customizable: StreamUpdate-extra-members />

    @classmethod
    def _parse(cls, msg: process_pb2.ProcessStreamMessage) -> 'StreamUpdate':
        pid = msg.pid
        fileno = msg.fileno
        chunk = msg.chunk
        return cls(pid, fileno, chunk)

    def _serialize(self, msg: process_pb2.ProcessStreamMessage) -> None:
        msg.pid = self.pid
        msg.fileno = self.fileno
        msg.chunk = self.chunk


@RequestMsg.message(process_pb2.ProcessSignalAction, 'process_signal_action', fields=('pid', 'signal',))
@dataclass(frozen=True, repr=False)
class SignalAction(SimpleMessage):
    pid: int
    signal: int

    def __post_init__(self):
        # <default GSL customizable: SignalAction-init-validation>
        pass
        # </GSL customizable: SignalAction-init-validation>

    # <default GSL customizable: SignalAction-extra-members />

    @classmethod
    def _parse(cls, msg: process_pb2.ProcessSignalAction) -> 'SignalAction':
        pid = msg.pid
        signal = msg.signal
        return cls(pid, signal)

    def _serialize(self, msg: process_pb2.ProcessSignalAction) -> None:
        msg.pid = self.pid
        msg.signal = self.signal


@ReplyMsg.message(process_pb2.ProcessExitUpdate, 'process_exit_update', fields=('pid', 'exit_code',))
@dataclass(frozen=True, repr=False)
class ExitUpdate(SimpleMessage):
    is_async = True

    pid: int
    exit_code: int

    def __post_init__(self):
        # <default GSL customizable: ExitUpdate-init-validation>
        pass
        # </GSL customizable: ExitUpdate-init-validation>

    # <default GSL customizable: ExitUpdate-extra-members />

    @classmethod
    def _parse(cls, msg: process_pb2.ProcessExitUpdate) -> 'ExitUpdate':
        pid = msg.pid
        exit_code = msg.exit_code
        return cls(pid, exit_code)

    def _serialize(self, msg: process_pb2.ProcessExitUpdate) -> None:
        msg.pid = self.pid
        msg.exit_code = self.exit_code



from typing import Any, Sequence, Union
from dataclasses import dataclass

from . import RequestMsg, ReplyMsg, Message, SimpleMessage
from hedgehog.protocol.proto import ack_pb2
from hedgehog.utils import protobuf

__all__ = ['Acknowledgement']

# <GSL customizable: module-header>
from hedgehog.protocol.proto.ack_pb2 import OK, UNKNOWN_COMMAND, INVALID_COMMAND, UNSUPPORTED_COMMAND, FAILED_COMMAND

__all__ += ['OK', 'UNKNOWN_COMMAND', 'INVALID_COMMAND', 'UNSUPPORTED_COMMAND', 'FAILED_COMMAND']
# </GSL customizable: module-header>


@ReplyMsg.message(ack_pb2.Acknowledgement, 'acknowledgement', fields=('code', 'message',))
@dataclass(frozen=True, repr=False)
class Acknowledgement(SimpleMessage):
    code: int = OK
    message: str = ''

    def __post_init__(self):
        # <default GSL customizable: Acknowledgement-init-validation>
        pass
        # </GSL customizable: Acknowledgement-init-validation>

    # <default GSL customizable: Acknowledgement-extra-members />

    @classmethod
    def _parse(cls, msg: ack_pb2.Acknowledgement) -> 'Acknowledgement':
        code = msg.code
        message = msg.message
        return cls(code, message=message)

    def _serialize(self, msg: ack_pb2.Acknowledgement) -> None:
        msg.code = self.code
        msg.message = self.message



from typing import Any, Sequence, Union
from dataclasses import dataclass

from . import RequestMsg, ReplyMsg, Message, SimpleMessage
from hedgehog.protocol.proto import io_pb2
from hedgehog.utils import protobuf

__all__ = ['Request', 'Reply', 'Subscribe', 'Update']

# <GSL customizable: module-header>
from hedgehog.protocol.proto.subscription_pb2 import Subscription

__all__ += ['Subscription']
# </GSL customizable: module-header>


@protobuf.message(io_pb2.AnalogMessage, 'analog_message', fields=('port',))
@dataclass(frozen=True, repr=False)
class Request(Message):
    port: int

    def __post_init__(self):
        # <default GSL customizable: Request-init-validation>
        pass
        # </GSL customizable: Request-init-validation>

    # <default GSL customizable: Request-extra-members />

    def _serialize(self, msg: io_pb2.AnalogMessage) -> None:
        msg.port = self.port


@protobuf.message(io_pb2.AnalogMessage, 'analog_message', fields=('port', 'value',))
@dataclass(frozen=True, repr=False)
class Reply(Message):
    port: int
    value: int

    def __post_init__(self):
        # <default GSL customizable: Reply-init-validation>
        pass
        # </GSL customizable: Reply-init-validation>

    # <default GSL customizable: Reply-extra-members />

    def _serialize(self, msg: io_pb2.AnalogMessage) -> None:
        msg.port = self.port
        msg.value = self.value


@protobuf.message(io_pb2.AnalogMessage, 'analog_message', fields=('port', 'subscription',))
@dataclass(frozen=True, repr=False)
class Subscribe(Message):
    port: int
    subscription: Subscription

    def __post_init__(self):
        # <default GSL customizable: Subscribe-init-validation>
        pass
        # </GSL customizable: Subscribe-init-validation>

    # <default GSL customizable: Subscribe-extra-members />

    def _serialize(self, msg: io_pb2.AnalogMessage) -> None:
        msg.port = self.port
        msg.subscription.CopyFrom(self.subscription)


@protobuf.message(io_pb2.AnalogMessage, 'analog_message', fields=('port', 'value', 'subscription',))
@dataclass(frozen=True, repr=False)
class Update(Message):
    is_async = True

    port: int
    value: int
    subscription: Subscription

    def __post_init__(self):
        # <default GSL customizable: Update-init-validation>
        pass
        # </GSL customizable: Update-init-validation>

    # <default GSL customizable: Update-extra-members />

    def _serialize(self, msg: io_pb2.AnalogMessage) -> None:
        msg.port = self.port
        msg.value = self.value
        msg.subscription.CopyFrom(self.subscription)


@RequestMsg.parser('analog_message')
def _parse_analog_message_request(msg: io_pb2.AnalogMessage) -> Union[Request, Subscribe]:
    port = msg.port
    value = msg.value
    subscription = msg.subscription if msg.HasField('subscription') else None
    # <GSL customizable: _parse_analog_message_request-return>
    if subscription is None:
        return Request(port)
    else:
        return Subscribe(port, subscription)
    # </GSL customizable: _parse_analog_message_request-return>


@ReplyMsg.parser('analog_message')
def _parse_analog_message_reply(msg: io_pb2.AnalogMessage) -> Union[Reply, Update]:
    port = msg.port
    value = msg.value
    subscription = msg.subscription if msg.HasField('subscription') else None
    # <GSL customizable: _parse_analog_message_reply-return>
    if subscription is None:
        return Reply(port, value)
    else:
        return Update(port, value, subscription)
    # </GSL customizable: _parse_analog_message_reply-return>



from typing import Any, Sequence, Union
from dataclasses import dataclass

from . import RequestMsg, ReplyMsg, Message, SimpleMessage
from hedgehog.protocol.proto import emergency_pb2
from hedgehog.utils import protobuf

__all__ = ['Action', 'Request', 'Reply', 'Subscribe', 'Update']

# <GSL customizable: module-header>
from hedgehog.protocol.proto.subscription_pb2 import Subscription
# </GSL customizable: module-header>


@RequestMsg.message(emergency_pb2.EmergencyAction, 'emergency_action', fields=('activate',))
@dataclass(frozen=True, repr=False)
class Action(SimpleMessage):
    activate: bool

    def __post_init__(self):
        # <default GSL customizable: Action-init-validation>
        pass
        # </GSL customizable: Action-init-validation>

    # <default GSL customizable: Action-extra-members />

    @classmethod
    def _parse(cls, msg: emergency_pb2.EmergencyAction) -> 'Action':
        activate = msg.activate
        return cls(activate)

    def _serialize(self, msg: emergency_pb2.EmergencyAction) -> None:
        msg.activate = self.activate


@protobuf.message(emergency_pb2.EmergencyMessage, 'emergency_message', fields=())
@dataclass(frozen=True, repr=False)
class Request(Message):

    def __post_init__(self):
        # <default GSL customizable: Request-init-validation>
        pass
        # </GSL customizable: Request-init-validation>

    # <default GSL customizable: Request-extra-members />

    def _serialize(self, msg: emergency_pb2.EmergencyMessage) -> None:
        msg.SetInParent()


@protobuf.message(emergency_pb2.EmergencyMessage, 'emergency_message', fields=('active',))
@dataclass(frozen=True, repr=False)
class Reply(Message):
    active: bool

    def __post_init__(self):
        # <default GSL customizable: Reply-init-validation>
        pass
        # </GSL customizable: Reply-init-validation>

    # <default GSL customizable: Reply-extra-members />

    def _serialize(self, msg: emergency_pb2.EmergencyMessage) -> None:
        msg.active = self.active


@protobuf.message(emergency_pb2.EmergencyMessage, 'emergency_message', fields=('subscription',))
@dataclass(frozen=True, repr=False)
class Subscribe(Message):
    subscription: Subscription

    def __post_init__(self):
        # <default GSL customizable: Subscribe-init-validation>
        pass
        # </GSL customizable: Subscribe-init-validation>

    # <default GSL customizable: Subscribe-extra-members />

    def _serialize(self, msg: emergency_pb2.EmergencyMessage) -> None:
        msg.subscription.CopyFrom(self.subscription)


@protobuf.message(emergency_pb2.EmergencyMessage, 'emergency_message', fields=('active', 'subscription',))
@dataclass(frozen=True, repr=False)
class Update(Message):
    is_async = True

    active: bool
    subscription: Subscription

    def __post_init__(self):
        # <default GSL customizable: Update-init-validation>
        pass
        # </GSL customizable: Update-init-validation>

    # <default GSL customizable: Update-extra-members />

    def _serialize(self, msg: emergency_pb2.EmergencyMessage) -> None:
        msg.active = self.active
        msg.subscription.CopyFrom(self.subscription)


@RequestMsg.parser('emergency_message')
def _parse_emergency_message_request(msg: emergency_pb2.EmergencyMessage) -> Union[Request, Subscribe]:
    active = msg.active
    subscription = msg.subscription if msg.HasField('subscription') else None
    # <GSL customizable: _parse_emergency_message_request-return>
    if subscription is None:
        return Request()
    else:
        return Subscribe(subscription)
    # </GSL customizable: _parse_emergency_message_request-return>


@ReplyMsg.parser('emergency_message')
def _parse_emergency_message_reply(msg: emergency_pb2.EmergencyMessage) -> Union[Reply, Update]:
    active = msg.active
    subscription = msg.subscription if msg.HasField('subscription') else None
    # <GSL customizable: _parse_emergency_message_reply-return>
    if subscription is None:
        return Reply(active)
    else:
        return Update(active, subscription)
    # </GSL customizable: _parse_emergency_message_reply-return>



import os.path

from gsl import lines, generate

from gsl_protocol.grammar.HedgehogProtocolVisitor import Field, Oneof, MandatoryParam, RepeatedParam, OptionalParam
from . import unique


def generate_code(model, root='.'):
    for mod in model.modules:
        generate_module_code(model, mod, root)


def generate_module_code(model, mod, root):
    out_file = os.path.join(root, 'hedgehog/protocol/messages', *mod.path, f'{mod.name}.py')
    os.makedirs(os.path.dirname(out_file), exist_ok=True)

    @generate(out_file)
    def code():
        def map_params(messageClass, mandatory, repeated, optional, custom=None, custom_optional=None):
            def has(attr_caller):
                try:
                    attr_caller()
                except AttributeError:
                    return False
                else:
                    return True

            for param in messageClass.params:
                if isinstance(param, MandatoryParam) and has(lambda: param.field.python_spec):
                    yield mandatory(param)
                elif isinstance(param, RepeatedParam) and has(lambda: param.field.python_spec):
                    yield repeated(param)
                elif isinstance(param, OptionalParam) and has(lambda: [field.python_spec for field in param.fields]):
                    for i in range(len(param.options)):
                        yield optional(param, i)
                elif isinstance(param, OptionalParam):
                    if not custom_optional:
                        raise RuntimeError(f"Unexpected custom optional parameter: {param}")
                    for i in range(len(param.options)):
                        yield custom_optional(param, i)
                else:
                    if not custom:
                        raise RuntimeError(f"Unexpected custom parameter: {param}")
                    yield custom(param)

        def map_params_code(messageClass, mandatory, repeated, optional, custom=None, custom_optional=None):
            for generator in map_params(messageClass, mandatory, repeated, optional, custom, custom_optional):
                yield from generator

        def field_names(messageClass):
            return map_params(
                messageClass,
                mandatory=lambda param: param.name,
                repeated=lambda param: param.name,
                optional=lambda param, i: param.options[i],
                custom=lambda param: param.name,
                custom_optional=lambda param, i: param.options[i],
            )

        def message_class_code(messageClass):
            message = messageClass.message
            proto = message.proto

            def message_fields_code():
                def field_str(name, typ, default=None, repeated=False, optional=False):
                    if repeated:
                        return f"    {name}: Sequence[{typ}]"
                    elif default is not None:
                        return f"    {name}: {typ} = {default}"
                    elif optional:
                        return f"    {name}: {typ} = None"
                    else:
                        return f"    {name}: {typ}"

                def mandatory(param):
                    python = param.field.python_spec
                    yield field_str(param.name, python.typ, python.default)

                def repeated(param):
                    python = param.field.python_spec
                    yield field_str(param.name, python.typ, python.default, repeated=True)

                def optional(param, i):
                    python = param.fields[i].python_spec
                    yield field_str(param.options[i], python.typ, python.default, optional=True)

                def custom(param):
                    yield field_str(param.name, 'Any')

                def custom_optional(param, i):
                    yield field_str(param.options[i], 'Any')

                yield from map_params_code(messageClass, mandatory, repeated, optional, custom, custom_optional)

            def message_init_code():
                # only generate an __init__ method if the dataclass __init__ is not enough
                if not any(isinstance(param, RepeatedParam) for param in messageClass.params):
                    return

                def init_param_strs():
                    def param_str(name, typ, default=None, repeated=False, optional=False):
                        if repeated:
                            return f"*{name}: {typ}"
                        elif default is not None:
                            return f"{name}: {typ}={default}"
                        elif optional:
                            return f"{name}: {typ}=None"
                        else:
                            return f"{name}: {typ}"

                    def mandatory(param):
                        python = param.field.python_spec
                        return param_str(param.name, python.typ, python.default)

                    def repeated(param):
                        python = param.field.python_spec
                        return param_str(param.name, python.typ, python.default, repeated=True)

                    def optional(param, i):
                        python = param.fields[i].python_spec
                        return param_str(param.options[i], python.typ, python.default, optional=True)

                    def custom(param):
                        return param.name

                    def custom_optional(param, i):
                        return param.options[i]

                    yield "self"
                    yield from map_params(messageClass, mandatory, repeated, optional, custom, custom_optional)

                yield from lines(f"""\

    def __init__({", ".join(init_param_strs())}) -> None:""")

                for name in field_names(messageClass):
                    yield from lines(f"""\
        object.__setattr__(self, {name!r}, {name})""")

                yield from lines(f"""\
        self.__post_init__()""")

            def message_parse_code():
                def init_param_strs():
                    yield from map_params(
                        messageClass,
                        mandatory=lambda param: param.name,
                        repeated=lambda param: f"*{param.name}",
                        optional=lambda param, i: f"{param.options[i]}={param.options[i]}",
                        custom=lambda param: param.name,
                        custom_optional=lambda param, i: param.options[i],
                    )

                yield from lines(f"""\

    @classmethod
    def _parse(cls, msg: {proto.name}_pb2.{message.name}) -> '{messageClass.name}':""")

                yield from map_params_code(
                    messageClass,
                    mandatory=lambda param: lines(f"""\
        {param.name} = msg.{param.name}"""),
                    repeated=lambda param: lines(f"""\
        {param.name} = msg.{param.name}"""),
                    optional=lambda param, i: lines(f"""\
        {param.options[i]} = msg.{param.options[i]}""")
                    if len(param.options) == 1 else lines(f"""\
        {param.options[i]} = msg.{param.options[i]} if msg.HasField('{param.options[i]}') else None"""),
                    custom=lambda param: lines(f"""\
        # <default GSL customizable: {messageClass.name}-parse-{param.name}>
        # TODO parse custom field '{param.name}'
        {param.name} = msg.{param.name}
        # </GSL customizable: {messageClass.name}-parse-{param.name}>"""),
                    custom_optional=lambda param, i: lines(f"""\
        # <default GSL customizable: {messageClass.name}-parse-{param.options[i]}>
        # TODO parse custom field '{param.options[i]}'
        {param.options[i]} = msg.{param.options[i]}
        # </GSL customizable: {messageClass.name}-parse-{param.options[i]}>"""),
                )

                yield from lines(f"""\
        return cls({", ".join(init_param_strs())})""")

            def message_serialize_code():
                yield from lines(f"""\

    def _serialize(self, msg: {proto.name}_pb2.{message.name}) -> None:""")

                def assignment_str(name, nested=False, repeated=False):
                    if repeated:
                        return f"msg.{name}.extend(self.{name})"
                    elif nested:
                        return f"msg.{name}.CopyFrom(self.{name})"
                    else:
                        return f"msg.{name} = self.{name}"

                if not messageClass.params:
                    yield from lines(f"""\
        msg.SetInParent()""")
                else:
                    yield from map_params_code(
                        messageClass,
                        mandatory=lambda param: lines(f"""\
        {assignment_str(param.name, nested=param.field.nested)}"""),
                        repeated=lambda param: lines(f"""\
        {assignment_str(param.name, nested=param.field.nested, repeated=True)}"""),
                        optional=lambda param, i: lines(f"""\
        {assignment_str(param.options[i], nested=param.fields[i].nested)}""")
                        if len(param.options) == 1 else lines(f"""\
        if self.{param.options[i]} is not None:
            {assignment_str(param.options[i], nested=param.fields[i].nested)}"""),
                        custom=lambda param: lines(f"""\
        # <default GSL customizable: {messageClass.name}-serialize-{param.name}>
        # TODO serialize custom field '{param.name}'
        {assignment_str(param.name)}
        # </GSL customizable: {messageClass.name}-serialize-{param.name}>"""),
                        custom_optional=lambda param, i: lines(f"""\
        # <default GSL customizable: {messageClass.name}-serialize-{param.options[i]}>
        # TODO serialize custom field '{param.options[i]}'
        {assignment_str(param.options[i])}
        # </GSL customizable: {messageClass.name}-serialize-{param.options[i]}>"""),
                    )

            request = messageClass.direction == "=>"
            is_async = messageClass.direction == "<-"
            complex = len(message.requestClasses if request else message.replyClasses) > 1

            decorator = "protobuf" if complex else "RequestMsg" if request else "ReplyMsg"

            fields_str = ' '.join(f'{field_name!r},' for field_name in field_names(messageClass))

            yield from lines(f"""\


@{decorator}.message({proto.name}_pb2.{message.name}, '{message.discriminator}', fields=({fields_str}))
@dataclass(frozen=True, repr=False)
class {messageClass.name}({"Message" if complex else "SimpleMessage"}):""")

            if is_async:
                yield from lines(f"""\
    is_async = True

""")

            yield from message_fields_code()
            yield from message_init_code()

            yield from lines(f"""\

    def __post_init__(self):
        # <default GSL customizable: {messageClass.name}-init-validation>
        pass
        # </GSL customizable: {messageClass.name}-init-validation>""")

            yield from lines(f"""\

    # <default GSL customizable: {messageClass.name}-extra-members />""")

            if not complex:
                yield from message_parse_code()

            yield from message_serialize_code()

        def complex_parser_code(message, request):
            def init_param_strs(messageClass):
                yield from map_params(
                    messageClass,
                    mandatory=lambda param: param.name,
                    repeated=lambda param: f"*{param.name}",
                    optional=lambda param, i: f"{param.options[i]}={param.options[i]}",
                    custom=lambda param: param.name,
                    custom_optional=lambda param, i: param.options[i],
                )

            messageClasses = message.requestClasses if request else message.replyClasses
            proto = message.proto

            decorator = "RequestMsg" if request else "ReplyMsg"
            method_name = f"_parse_{message.discriminator}_{'request' if request else 'reply'}"

            yield from lines(f"""\


@{decorator}.parser('{message.discriminator}')
def {method_name}(msg: {proto.name}_pb2.{message.name}) \
-> Union[{', '.join(messageClass.name for messageClass in messageClasses)}]:""")

            for field in message.fields:
                if isinstance(field, Field):
                    if not field.nested:
                        yield from lines(f"""\
    {field.name} = msg.{field.name}""")
                    else:
                        yield from lines(f"""\
    {field.name} = msg.{field.name} if msg.HasField('{field.name}') else None""")
                elif isinstance(field, Oneof):
                    for field in field.fields:
                        yield from lines(f"""\
    {field.name} = msg.{field.name} if msg.HasField('{field.name}') else None""")
                else:
                    assert False

            yield from lines(f"""\
    # <default GSL customizable: {method_name}-return>
    # TODO return correct message instance""")

            for messageClass in messageClasses:
                yield from lines(f"""\
    #return {messageClass.name}({", ".join(init_param_strs(messageClass))})""")

            yield from lines(f"""\
    return None
    # </GSL customizable: {method_name}-return>""")

        yield from lines(f"""\
from typing import Any, Sequence, Union
from dataclasses import dataclass

from {'.' * (len(mod.path) + 1)} import RequestMsg, ReplyMsg, Message, SimpleMessage""")
        for protoPath, protoName in unique((proto.path, proto.name)
                                           for messageClass in mod.messageClasses
                                           for proto in (messageClass.message.proto,)):
            yield from lines(f"""\
from hedgehog.protocol.proto{'.'.join(('',) + protoPath)} import {protoName}_pb2""")
        yield from lines(f"""\
from hedgehog.utils import protobuf

__all__ = [{', '.join(repr(messageClass.name) for messageClass in mod.messageClasses)}]

# <default GSL customizable: module-header />""")
        for messageClass in mod.messageClasses:
            yield from message_class_code(messageClass)
        for message in mod.complexMessages:
            if len(message.requestClasses) > 1:
                yield from complex_parser_code(message, request=True)
            if len(message.replyClasses) > 1:
                yield from complex_parser_code(message, request=False)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: hedgehog/protocol/proto/emergency.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from hedgehog.protocol.proto import subscription_pb2 as hedgehog_dot_protocol_dot_proto_dot_subscription__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='hedgehog/protocol/proto/emergency.proto',
  package='hedgehog.protocol.proto',
  syntax='proto3',
  serialized_pb=_b('\n\'hedgehog/protocol/proto/emergency.proto\x12\x17hedgehog.protocol.proto\x1a*hedgehog/protocol/proto/subscription.proto\"#\n\x0f\x45mergencyAction\x12\x10\n\x08\x61\x63tivate\x18\x01 \x01(\x08\"_\n\x10\x45mergencyMessage\x12\x0e\n\x06\x61\x63tive\x18\x01 \x01(\x08\x12;\n\x0csubscription\x18\x02 \x01(\x0b\x32%.hedgehog.protocol.proto.SubscriptionB-\n\x1f\x61t.pria.hedgehog.protocol.protoB\nEmergencyPb\x06proto3')
  ,
  dependencies=[hedgehog_dot_protocol_dot_proto_dot_subscription__pb2.DESCRIPTOR,])




_EMERGENCYACTION = _descriptor.Descriptor(
  name='EmergencyAction',
  full_name='hedgehog.protocol.proto.EmergencyAction',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='activate', full_name='hedgehog.protocol.proto.EmergencyAction.activate', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=112,
  serialized_end=147,
)


_EMERGENCYMESSAGE = _descriptor.Descriptor(
  name='EmergencyMessage',
  full_name='hedgehog.protocol.proto.EmergencyMessage',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='active', full_name='hedgehog.protocol.proto.EmergencyMessage.active', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='subscription', full_name='hedgehog.protocol.proto.EmergencyMessage.subscription', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=149,
  serialized_end=244,
)

_EMERGENCYMESSAGE.fields_by_name['subscription'].message_type = hedgehog_dot_protocol_dot_proto_dot_subscription__pb2._SUBSCRIPTION
DESCRIPTOR.message_types_by_name['EmergencyAction'] = _EMERGENCYACTION
DESCRIPTOR.message_types_by_name['EmergencyMessage'] = _EMERGENCYMESSAGE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

EmergencyAction = _reflection.GeneratedProtocolMessageType('EmergencyAction', (_message.Message,), dict(
  DESCRIPTOR = _EMERGENCYACTION,
  __module__ = 'hedgehog.protocol.proto.emergency_pb2'
  # @@protoc_insertion_point(class_scope:hedgehog.protocol.proto.EmergencyAction)
  ))
_sym_db.RegisterMessage(EmergencyAction)

EmergencyMessage = _reflection.GeneratedProtocolMessageType('EmergencyMessage', (_message.Message,), dict(
  DESCRIPTOR = _EMERGENCYMESSAGE,
  __module__ = 'hedgehog.protocol.proto.emergency_pb2'
  # @@protoc_insertion_point(class_scope:hedgehog.protocol.proto.EmergencyMessage)
  ))
_sym_db.RegisterMessage(EmergencyMessage)


DESCRIPTOR.has_options = True
DESCRIPTOR._options = _descriptor._ParseOptions(descriptor_pb2.FileOptions(), _b('\n\037at.pria.hedgehog.protocol.protoB\nEmergencyP'))
# @@protoc_insertion_point(module_scope)



# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: hedgehog/protocol/proto/vision.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='hedgehog/protocol/proto/vision.proto',
  package='hedgehog.protocol.proto',
  syntax='proto3',
  serialized_pb=_b('\n$hedgehog/protocol/proto/vision.proto\x12\x17hedgehog.protocol.proto\"\x0e\n\x0c\x46\x61\x63\x65sChannel\"0\n\x0c\x42lobsChannel\x12\x0f\n\x07hsv_min\x18\x01 \x01(\x07\x12\x0f\n\x07hsv_max\x18\x02 \x01(\x07\"\x91\x01\n\x07\x43hannel\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x36\n\x05\x66\x61\x63\x65s\x18\x02 \x01(\x0b\x32%.hedgehog.protocol.proto.FacesChannelH\x00\x12\x36\n\x05\x62lobs\x18\x03 \x01(\x0b\x32%.hedgehog.protocol.proto.BlobsChannelH\x00\x42\t\n\x07\x63hannel\";\n\x04\x46\x61\x63\x65\x12\t\n\x01x\x18\x01 \x01(\r\x12\t\n\x01y\x18\x02 \x01(\r\x12\r\n\x05width\x18\x03 \x01(\r\x12\x0e\n\x06height\x18\x04 \x01(\r\"<\n\x0c\x46\x61\x63\x65sFeature\x12,\n\x05\x66\x61\x63\x65s\x18\x01 \x03(\x0b\x32\x1d.hedgehog.protocol.proto.Face\"g\n\x04\x42lob\x12\t\n\x01x\x18\x01 \x01(\r\x12\t\n\x01y\x18\x02 \x01(\r\x12\r\n\x05width\x18\x03 \x01(\r\x12\x0e\n\x06height\x18\x04 \x01(\r\x12\n\n\x02\x63x\x18\x05 \x01(\r\x12\n\n\x02\x63y\x18\x06 \x01(\r\x12\x12\n\nconfidence\x18\x07 \x01(\x02\"<\n\x0c\x42lobsFeature\x12,\n\x05\x62lobs\x18\x01 \x03(\x0b\x32\x1d.hedgehog.protocol.proto.Blob\"\x84\x01\n\x07\x46\x65\x61ture\x12\x36\n\x05\x66\x61\x63\x65s\x18\x01 \x01(\x0b\x32%.hedgehog.protocol.proto.FacesFeatureH\x00\x12\x36\n\x05\x62lobs\x18\x02 \x01(\x0b\x32%.hedgehog.protocol.proto.BlobsFeatureH\x00\x42\t\n\x07\x66\x65\x61ture\"\"\n\x12VisionCameraAction\x12\x0c\n\x04open\x18\x01 \x01(\x08\"\x81\x01\n\x14VisionChannelMessage\x12\x35\n\x02op\x18\x01 \x01(\x0e\x32).hedgehog.protocol.proto.ChannelOperation\x12\x32\n\x08\x63hannels\x18\x02 \x03(\x0b\x32 .hedgehog.protocol.proto.Channel\"\x1a\n\x18VisionCaptureFrameAction\"6\n\x12VisionFrameMessage\x12\x11\n\thighlight\x18\x01 \x01(\t\x12\r\n\x05\x66rame\x18\x02 \x01(\x0c\"Z\n\x14VisionFeatureMessage\x12\x0f\n\x07\x63hannel\x18\x01 \x01(\t\x12\x31\n\x07\x66\x65\x61ture\x18\x02 \x01(\x0b\x32 .hedgehog.protocol.proto.Feature*@\n\x10\x43hannelOperation\x12\x08\n\x04READ\x10\x00\x12\n\n\x06\x43REATE\x10\x01\x12\n\n\x06UPDATE\x10\x02\x12\n\n\x06\x44\x45LETE\x10\x03\x42*\n\x1f\x61t.pria.hedgehog.protocol.protoB\x07VisionPb\x06proto3')
)

_CHANNELOPERATION = _descriptor.EnumDescriptor(
  name='ChannelOperation',
  full_name='hedgehog.protocol.proto.ChannelOperation',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='READ', index=0, number=0,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='CREATE', index=1, number=1,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='UPDATE', index=2, number=2,
      options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='DELETE', index=3, number=3,
      options=None,
      type=None),
  ],
  containing_type=None,
  options=None,
  serialized_start=1048,
  serialized_end=1112,
)
_sym_db.RegisterEnumDescriptor(_CHANNELOPERATION)

ChannelOperation = enum_type_wrapper.EnumTypeWrapper(_CHANNELOPERATION)
READ = 0
CREATE = 1
UPDATE = 2
DELETE = 3



_FACESCHANNEL = _descriptor.Descriptor(
  name='FacesChannel',
  full_name='hedgehog.protocol.proto.FacesChannel',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=65,
  serialized_end=79,
)


_BLOBSCHANNEL = _descriptor.Descriptor(
  name='BlobsChannel',
  full_name='hedgehog.protocol.proto.BlobsChannel',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='hsv_min', full_name='hedgehog.protocol.proto.BlobsChannel.hsv_min', index=0,
      number=1, type=7, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='hsv_max', full_name='hedgehog.protocol.proto.BlobsChannel.hsv_max', index=1,
      number=2, type=7, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=81,
  serialized_end=129,
)


_CHANNEL = _descriptor.Descriptor(
  name='Channel',
  full_name='hedgehog.protocol.proto.Channel',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='hedgehog.protocol.proto.Channel.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='faces', full_name='hedgehog.protocol.proto.Channel.faces', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='blobs', full_name='hedgehog.protocol.proto.Channel.blobs', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='channel', full_name='hedgehog.protocol.proto.Channel.channel',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=132,
  serialized_end=277,
)


_FACE = _descriptor.Descriptor(
  name='Face',
  full_name='hedgehog.protocol.proto.Face',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='x', full_name='hedgehog.protocol.proto.Face.x', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='y', full_name='hedgehog.protocol.proto.Face.y', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='width', full_name='hedgehog.protocol.proto.Face.width', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='height', full_name='hedgehog.protocol.proto.Face.height', index=3,
      number=4, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=279,
  serialized_end=338,
)


_FACESFEATURE = _descriptor.Descriptor(
  name='FacesFeature',
  full_name='hedgehog.protocol.proto.FacesFeature',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='faces', full_name='hedgehog.protocol.proto.FacesFeature.faces', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=340,
  serialized_end=400,
)


_BLOB = _descriptor.Descriptor(
  name='Blob',
  full_name='hedgehog.protocol.proto.Blob',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='x', full_name='hedgehog.protocol.proto.Blob.x', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='y', full_name='hedgehog.protocol.proto.Blob.y', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='width', full_name='hedgehog.protocol.proto.Blob.width', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='height', full_name='hedgehog.protocol.proto.Blob.height', index=3,
      number=4, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='cx', full_name='hedgehog.protocol.proto.Blob.cx', index=4,
      number=5, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='cy', full_name='hedgehog.protocol.proto.Blob.cy', index=5,
      number=6, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='confidence', full_name='hedgehog.protocol.proto.Blob.confidence', index=6,
      number=7, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=402,
  serialized_end=505,
)


_BLOBSFEATURE = _descriptor.Descriptor(
  name='BlobsFeature',
  full_name='hedgehog.protocol.proto.BlobsFeature',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='blobs', full_name='hedgehog.protocol.proto.BlobsFeature.blobs', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=507,
  serialized_end=567,
)


_FEATURE = _descriptor.Descriptor(
  name='Feature',
  full_name='hedgehog.protocol.proto.Feature',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='faces', full_name='hedgehog.protocol.proto.Feature.faces', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='blobs', full_name='hedgehog.protocol.proto.Feature.blobs', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='feature', full_name='hedgehog.protocol.proto.Feature.feature',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=570,
  serialized_end=702,
)


_VISIONCAMERAACTION = _descriptor.Descriptor(
  name='VisionCameraAction',
  full_name='hedgehog.protocol.proto.VisionCameraAction',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='open', full_name='hedgehog.protocol.proto.VisionCameraAction.open', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=704,
  serialized_end=738,
)


_VISIONCHANNELMESSAGE = _descriptor.Descriptor(
  name='VisionChannelMessage',
  full_name='hedgehog.protocol.proto.VisionChannelMessage',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='op', full_name='hedgehog.protocol.proto.VisionChannelMessage.op', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='channels', full_name='hedgehog.protocol.proto.VisionChannelMessage.channels', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=741,
  serialized_end=870,
)


_VISIONCAPTUREFRAMEACTION = _descriptor.Descriptor(
  name='VisionCaptureFrameAction',
  full_name='hedgehog.protocol.proto.VisionCaptureFrameAction',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=872,
  serialized_end=898,
)


_VISIONFRAMEMESSAGE = _descriptor.Descriptor(
  name='VisionFrameMessage',
  full_name='hedgehog.protocol.proto.VisionFrameMessage',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='highlight', full_name='hedgehog.protocol.proto.VisionFrameMessage.highlight', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='frame', full_name='hedgehog.protocol.proto.VisionFrameMessage.frame', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=900,
  serialized_end=954,
)


_VISIONFEATUREMESSAGE = _descriptor.Descriptor(
  name='VisionFeatureMessage',
  full_name='hedgehog.protocol.proto.VisionFeatureMessage',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='channel', full_name='hedgehog.protocol.proto.VisionFeatureMessage.channel', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='feature', full_name='hedgehog.protocol.proto.VisionFeatureMessage.feature', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=956,
  serialized_end=1046,
)

_CHANNEL.fields_by_name['faces'].message_type = _FACESCHANNEL
_CHANNEL.fields_by_name['blobs'].message_type = _BLOBSCHANNEL
_CHANNEL.oneofs_by_name['channel'].fields.append(
  _CHANNEL.fields_by_name['faces'])
_CHANNEL.fields_by_name['faces'].containing_oneof = _CHANNEL.oneofs_by_name['channel']
_CHANNEL.oneofs_by_name['channel'].fields.append(
  _CHANNEL.fields_by_name['blobs'])
_CHANNEL.fields_by_name['blobs'].containing_oneof = _CHANNEL.oneofs_by_name['channel']
_FACESFEATURE.fields_by_name['faces'].message_type = _FACE
_BLOBSFEATURE.fields_by_name['blobs'].message_type = _BLOB
_FEATURE.fields_by_name['faces'].message_type = _FACESFEATURE
_FEATURE.fields_by_name['blobs'].message_type = _BLOBSFEATURE
_FEATURE.oneofs_by_name['feature'].fields.append(
  _FEATURE.fields_by_name['faces'])
_FEATURE.fields_by_name['faces'].containing_oneof = _FEATURE.oneofs_by_name['feature']
_FEATURE.oneofs_by_name['feature'].fields.append(
  _FEATURE.fields_by_name['blobs'])
_FEATURE.fields_by_name['blobs'].containing_oneof = _FEATURE.oneofs_by_name['feature']
_VISIONCHANNELMESSAGE.fields_by_name['op'].enum_type = _CHANNELOPERATION
_VISIONCHANNELMESSAGE.fields_by_name['channels'].message_type = _CHANNEL
_VISIONFEATUREMESSAGE.fields_by_name['feature'].message_type = _FEATURE
DESCRIPTOR.message_types_by_name['FacesChannel'] = _FACESCHANNEL
DESCRIPTOR.message_types_by_name['BlobsChannel'] = _BLOBSCHANNEL
DESCRIPTOR.message_types_by_name['Channel'] = _CHANNEL
DESCRIPTOR.message_types_by_name['Face'] = _FACE
DESCRIPTOR.message_types_by_name['FacesFeature'] = _FACESFEATURE
DESCRIPTOR.message_types_by_name['Blob'] = _BLOB
DESCRIPTOR.message_types_by_name['BlobsFeature'] = _BLOBSFEATURE
DESCRIPTOR.message_types_by_name['Feature'] = _FEATURE
DESCRIPTOR.message_types_by_name['VisionCameraAction'] = _VISIONCAMERAACTION
DESCRIPTOR.message_types_by_name['VisionChannelMessage'] = _VISIONCHANNELMESSAGE
DESCRIPTOR.message_types_by_name['VisionCaptureFrameAction'] = _VISIONCAPTUREFRAMEACTION
DESCRIPTOR.message_types_by_name['VisionFrameMessage'] = _VISIONFRAMEMESSAGE
DESCRIPTOR.message_types_by_name['VisionFeatureMessage'] = _VISIONFEATUREMESSAGE
DESCRIPTOR.enum_types_by_name['ChannelOperation'] = _CHANNELOPERATION
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

FacesChannel = _reflection.GeneratedProtocolMessageType('FacesChannel', (_message.Message,), dict(
  DESCRIPTOR = _FACESCHANNEL,
  __module__ = 'hedgehog.protocol.proto.vision_pb2'
  # @@protoc_insertion_point(class_scope:hedgehog.protocol.proto.FacesChannel)
  ))
_sym_db.RegisterMessage(FacesChannel)

BlobsChannel = _reflection.GeneratedProtocolMessageType('BlobsChannel', (_message.Message,), dict(
  DESCRIPTOR = _BLOBSCHANNEL,
  __module__ = 'hedgehog.protocol.proto.vision_pb2'
  # @@protoc_insertion_point(class_scope:hedgehog.protocol.proto.BlobsChannel)
  ))
_sym_db.RegisterMessage(BlobsChannel)

Channel = _reflection.GeneratedProtocolMessageType('Channel', (_message.Message,), dict(
  DESCRIPTOR = _CHANNEL,
  __module__ = 'hedgehog.protocol.proto.vision_pb2'
  # @@protoc_insertion_point(class_scope:hedgehog.protocol.proto.Channel)
  ))
_sym_db.RegisterMessage(Channel)

Face = _reflection.GeneratedProtocolMessageType('Face', (_message.Message,), dict(
  DESCRIPTOR = _FACE,
  __module__ = 'hedgehog.protocol.proto.vision_pb2'
  # @@protoc_insertion_point(class_scope:hedgehog.protocol.proto.Face)
  ))
_sym_db.RegisterMessage(Face)

FacesFeature = _reflection.GeneratedProtocolMessageType('FacesFeature', (_message.Message,), dict(
  DESCRIPTOR = _FACESFEATURE,
  __module__ = 'hedgehog.protocol.proto.vision_pb2'
  # @@protoc_insertion_point(class_scope:hedgehog.protocol.proto.FacesFeature)
  ))
_sym_db.RegisterMessage(FacesFeature)

Blob = _reflection.GeneratedProtocolMessageType('Blob', (_message.Message,), dict(
  DESCRIPTOR = _BLOB,
  __module__ = 'hedgehog.protocol.proto.vision_pb2'
  # @@protoc_insertion_point(class_scope:hedgehog.protocol.proto.Blob)
  ))
_sym_db.RegisterMessage(Blob)

BlobsFeature = _reflection.GeneratedProtocolMessageType('BlobsFeature', (_message.Message,), dict(
  DESCRIPTOR = _BLOBSFEATURE,
  __module__ = 'hedgehog.protocol.proto.vision_pb2'
  # @@protoc_insertion_point(class_scope:hedgehog.protocol.proto.BlobsFeature)
  ))
_sym_db.RegisterMessage(BlobsFeature)

Feature = _reflection.GeneratedProtocolMessageType('Feature', (_message.Message,), dict(
  DESCRIPTOR = _FEATURE,
  __module__ = 'hedgehog.protocol.proto.vision_pb2'
  # @@protoc_insertion_point(class_scope:hedgehog.protocol.proto.Feature)
  ))
_sym_db.RegisterMessage(Feature)

VisionCameraAction = _reflection.GeneratedProtocolMessageType('VisionCameraAction', (_message.Message,), dict(
  DESCRIPTOR = _VISIONCAMERAACTION,
  __module__ = 'hedgehog.protocol.proto.vision_pb2'
  # @@protoc_insertion_point(class_scope:hedgehog.protocol.proto.VisionCameraAction)
  ))
_sym_db.RegisterMessage(VisionCameraAction)

VisionChannelMessage = _reflection.GeneratedProtocolMessageType('VisionChannelMessage', (_message.Message,), dict(
  DESCRIPTOR = _VISIONCHANNELMESSAGE,
  __module__ = 'hedgehog.protocol.proto.vision_pb2'
  # @@protoc_insertion_point(class_scope:hedgehog.protocol.proto.VisionChannelMessage)
  ))
_sym_db.RegisterMessage(VisionChannelMessage)

VisionCaptureFrameAction = _reflection.GeneratedProtocolMessageType('VisionCaptureFrameAction', (_message.Message,), dict(
  DESCRIPTOR = _VISIONCAPTUREFRAMEACTION,
  __module__ = 'hedgehog.protocol.proto.vision_pb2'
  # @@protoc_insertion_point(class_scope:hedgehog.protocol.proto.VisionCaptureFrameAction)
  ))
_sym_db.RegisterMessage(VisionCaptureFrameAction)

VisionFrameMessage = _reflection.GeneratedProtocolMessageType('VisionFrameMessage', (_message.Message,), dict(
  DESCRIPTOR = _VISIONFRAMEMESSAGE,
  __module__ = 'hedgehog.protocol.proto.vision_pb2'
  # @@protoc_insertion_point(class_scope:hedgehog.protocol.proto.VisionFrameMessage)
  ))
_sym_db.RegisterMessage(VisionFrameMessage)

VisionFeatureMessage = _reflection.GeneratedProtocolMessageType('VisionFeatureMessage', (_message.Message,), dict(
  DESCRIPTOR = _VISIONFEATUREMESSAGE,
  __module__ = 'hedgehog.protocol.proto.vision_pb2'
  # @@protoc_insertion_point(class_scope:hedgehog.protocol.proto.VisionFeatureMessage)
  ))
_sym_db.RegisterMessage(VisionFeatureMessage)


DESCRIPTOR.has_options = True
DESCRIPTOR._options = _descriptor._ParseOptions(descriptor_pb2.FileOptions(), _b('\n\037at.pria.hedgehog.protocol.protoB\007VisionP'))
# @@protoc_insertion_point(module_scope)



import os.path

from gsl.yaml import YAML

from gsl_protocol import proto_target, get_model as _get_model
from gsl_protocol.grammar.HedgehogProtocolVisitor import Oneof, Field

PYTHON_PROTOCOL_MODEL = os.path.join(os.path.dirname(__file__), 'python.yaml')


def unique(it):
    items = set()
    for item in it:
        if item not in items:
            items.add(item)
            yield item


def get_model(model_file=None, py_model_file=None):
    if py_model_file is None:
        py_model_file = PYTHON_PROTOCOL_MODEL

    with open(py_model_file) as f:
        yaml = YAML(typ='safe')
        py_model = yaml.load(f)

    model = _get_model(model_file)

    def get_fields(message):
        for field in message.fields:
            if isinstance(field, Field):
                yield field
            elif isinstance(field, Oneof):
                yield from field.fields

    for message in model.messages:
        full_name = '.'.join(message.qualifiedName.full_name)
        py_fields = py_model.get(full_name, None)
        if py_fields:
            for field in get_fields(message):
                py_field = py_fields.get(field.name, None)
                if py_field:
                    if 'default' not in py_field:
                        py_field.default = None
                    field.python_spec = py_field

    return model


def main():
    from . import python_target

    model = get_model()
    root = '.'
    proto_target.generate_code(model, root)
    python_target.generate_code(model, root)



from typing import Any, Optional, Sequence, Union
from dataclasses import dataclass

from . import RequestMsg, ReplyMsg, Message, SimpleMessage
from hedgehog.protocol.proto import servo_pb2
from hedgehog.utils import protobuf

__all__ = ['Action', 'CommandRequest', 'CommandReply', 'CommandSubscribe', 'CommandUpdate']

# <GSL customizable: module-header>
from hedgehog.protocol.errors import InvalidCommandError
from hedgehog.protocol.proto.subscription_pb2 import Subscription

__all__ += ['Subscription']
# </GSL customizable: module-header>


@RequestMsg.message(servo_pb2.ServoAction, 'servo_action', fields=('port', 'position',))
@dataclass(frozen=True, repr=False)
class Action(SimpleMessage):
    port: int
    position: Optional[int]

    def __post_init__(self):
        # <default GSL customizable: Action-init-validation>
        pass
        # </GSL customizable: Action-init-validation>

    # <default GSL customizable: Action-extra-members />

    @classmethod
    def _parse(cls, msg: servo_pb2.ServoAction) -> 'Action':
        port = msg.port
        active = msg.active
        position = msg.position
        return cls(port, position if active else None)

    def _serialize(self, msg: servo_pb2.ServoAction) -> None:
        msg.port = self.port
        msg.active = self.position is not None
        if self.position is not None:
            msg.position = self.position


@protobuf.message(servo_pb2.ServoCommandMessage, 'servo_command_message', fields=('port',))
@dataclass(frozen=True, repr=False)
class CommandRequest(Message):
    port: int

    def __post_init__(self):
        # <default GSL customizable: CommandRequest-init-validation>
        pass
        # </GSL customizable: CommandRequest-init-validation>

    # <default GSL customizable: CommandRequest-extra-members />

    def _serialize(self, msg: servo_pb2.ServoCommandMessage) -> None:
        msg.port = self.port


@protobuf.message(servo_pb2.ServoCommandMessage, 'servo_command_message', fields=('port', 'position',))
@dataclass(frozen=True, repr=False)
class CommandReply(Message):
    port: int
    position: Optional[int]

    def __post_init__(self):
        # <default GSL customizable: CommandReply-init-validation>
        pass
        # </GSL customizable: CommandReply-init-validation>

    # <default GSL customizable: CommandReply-extra-members />

    def _serialize(self, msg: servo_pb2.ServoCommandMessage) -> None:
        msg.port = self.port
        msg.active = self.position is not None
        if self.position is not None:
            msg.position = self.position


@protobuf.message(servo_pb2.ServoCommandMessage, 'servo_command_message', fields=('port', 'subscription',))
@dataclass(frozen=True, repr=False)
class CommandSubscribe(Message):
    port: int
    subscription: Subscription

    def __post_init__(self):
        # <default GSL customizable: CommandSubscribe-init-validation>
        pass
        # </GSL customizable: CommandSubscribe-init-validation>

    # <default GSL customizable: CommandSubscribe-extra-members />

    def _serialize(self, msg: servo_pb2.ServoCommandMessage) -> None:
        msg.port = self.port
        msg.subscription.CopyFrom(self.subscription)


@protobuf.message(servo_pb2.ServoCommandMessage, 'servo_command_message', fields=('port', 'position', 'subscription',))
@dataclass(frozen=True, repr=False)
class CommandUpdate(Message):
    is_async = True

    port: int
    position: Optional[int]
    subscription: Subscription

    def __post_init__(self):
        # <default GSL customizable: CommandUpdate-init-validation>
        pass
        # </GSL customizable: CommandUpdate-init-validation>

    # <default GSL customizable: CommandUpdate-extra-members />

    def _serialize(self, msg: servo_pb2.ServoCommandMessage) -> None:
        msg.port = self.port
        msg.active = self.position is not None
        if self.position is not None:
            msg.position = self.position
        msg.subscription.CopyFrom(self.subscription)


@RequestMsg.parser('servo_command_message')
def _parse_servo_command_message_request(msg: servo_pb2.ServoCommandMessage) -> Union[CommandRequest, CommandSubscribe]:
    port = msg.port
    active = msg.active
    position = msg.position
    subscription = msg.subscription if msg.HasField('subscription') else None
    # <GSL customizable: _parse_servo_command_message_request-return>
    if subscription is None:
        return CommandRequest(port)
    else:
        return CommandSubscribe(port, subscription)
    # </GSL customizable: _parse_servo_command_message_request-return>


@ReplyMsg.parser('servo_command_message')
def _parse_servo_command_message_reply(msg: servo_pb2.ServoCommandMessage) -> Union[CommandReply, CommandUpdate]:
    port = msg.port
    active = msg.active
    position = msg.position
    subscription = msg.subscription if msg.HasField('subscription') else None
    # <GSL customizable: _parse_servo_command_message_reply-return>
    if subscription is None:
        return CommandReply(port, position if active else None)
    else:
        return CommandUpdate(port, position if active else None, subscription)
    # </GSL customizable: _parse_servo_command_message_reply-return>



from typing import Any, Sequence, Union
from dataclasses import dataclass

from . import RequestMsg, ReplyMsg, Message, SimpleMessage
from hedgehog.protocol.proto import imu_pb2
from hedgehog.utils import protobuf

__all__ = ['RateRequest', 'RateReply', 'RateSubscribe', 'RateUpdate', 'AccelerationRequest', 'AccelerationReply', 'AccelerationSubscribe', 'AccelerationUpdate', 'PoseRequest', 'PoseReply', 'PoseSubscribe', 'PoseUpdate']

# <GSL customizable: module-header>
from hedgehog.protocol.proto.imu_pb2 import RATE, ACCELERATION, POSE
from hedgehog.protocol.proto.subscription_pb2 import Subscription

__all__ += [
    'RATE', 'ACCELERATION', 'POSE',
    'Subscription',
]
# </GSL customizable: module-header>


@protobuf.message(imu_pb2.ImuMessage, 'imu_message', fields=())
@dataclass(frozen=True, repr=False)
class RateRequest(Message):

    def __post_init__(self):
        # <default GSL customizable: RateRequest-init-validation>
        pass
        # </GSL customizable: RateRequest-init-validation>

    # <default GSL customizable: RateRequest-extra-members />

    def _serialize(self, msg: imu_pb2.ImuMessage) -> None:
        msg.kind = RATE


@protobuf.message(imu_pb2.ImuMessage, 'imu_message', fields=('x', 'y', 'z',))
@dataclass(frozen=True, repr=False)
class RateReply(Message):
    x: int
    y: int
    z: int

    def __post_init__(self):
        # <default GSL customizable: RateReply-init-validation>
        pass
        # </GSL customizable: RateReply-init-validation>

    # <default GSL customizable: RateReply-extra-members />

    def _serialize(self, msg: imu_pb2.ImuMessage) -> None:
        msg.kind = RATE
        msg.x = self.x
        msg.y = self.y
        msg.z = self.z


@protobuf.message(imu_pb2.ImuMessage, 'imu_message', fields=('subscription',))
@dataclass(frozen=True, repr=False)
class RateSubscribe(Message):
    subscription: Subscription

    def __post_init__(self):
        # <default GSL customizable: RateSubscribe-init-validation>
        pass
        # </GSL customizable: RateSubscribe-init-validation>

    # <default GSL customizable: RateSubscribe-extra-members />

    def _serialize(self, msg: imu_pb2.ImuMessage) -> None:
        msg.kind = RATE
        msg.subscription.CopyFrom(self.subscription)


@protobuf.message(imu_pb2.ImuMessage, 'imu_message', fields=('x', 'y', 'z', 'subscription',))
@dataclass(frozen=True, repr=False)
class RateUpdate(Message):
    is_async = True

    x: int
    y: int
    z: int
    subscription: Subscription

    def __post_init__(self):
        # <default GSL customizable: RateUpdate-init-validation>
        pass
        # </GSL customizable: RateUpdate-init-validation>

    # <default GSL customizable: RateUpdate-extra-members />

    def _serialize(self, msg: imu_pb2.ImuMessage) -> None:
        msg.kind = RATE
        msg.x = self.x
        msg.y = self.y
        msg.z = self.z
        msg.subscription.CopyFrom(self.subscription)


@protobuf.message(imu_pb2.ImuMessage, 'imu_message', fields=())
@dataclass(frozen=True, repr=False)
class AccelerationRequest(Message):

    def __post_init__(self):
        # <default GSL customizable: AccelerationRequest-init-validation>
        pass
        # </GSL customizable: AccelerationRequest-init-validation>

    # <default GSL customizable: AccelerationRequest-extra-members />

    def _serialize(self, msg: imu_pb2.ImuMessage) -> None:
        msg.kind = ACCELERATION


@protobuf.message(imu_pb2.ImuMessage, 'imu_message', fields=('x', 'y', 'z',))
@dataclass(frozen=True, repr=False)
class AccelerationReply(Message):
    x: int
    y: int
    z: int

    def __post_init__(self):
        # <default GSL customizable: AccelerationReply-init-validation>
        pass
        # </GSL customizable: AccelerationReply-init-validation>

    # <default GSL customizable: AccelerationReply-extra-members />

    def _serialize(self, msg: imu_pb2.ImuMessage) -> None:
        msg.kind = ACCELERATION
        msg.x = self.x
        msg.y = self.y
        msg.z = self.z


@protobuf.message(imu_pb2.ImuMessage, 'imu_message', fields=('subscription',))
@dataclass(frozen=True, repr=False)
class AccelerationSubscribe(Message):
    subscription: Subscription

    def __post_init__(self):
        # <default GSL customizable: AccelerationSubscribe-init-validation>
        pass
        # </GSL customizable: AccelerationSubscribe-init-validation>

    # <default GSL customizable: AccelerationSubscribe-extra-members />

    def _serialize(self, msg: imu_pb2.ImuMessage) -> None:
        msg.kind = ACCELERATION
        msg.subscription.CopyFrom(self.subscription)


@protobuf.message(imu_pb2.ImuMessage, 'imu_message', fields=('x', 'y', 'z', 'subscription',))
@dataclass(frozen=True, repr=False)
class AccelerationUpdate(Message):
    is_async = True

    x: int
    y: int
    z: int
    subscription: Subscription

    def __post_init__(self):
        # <default GSL customizable: AccelerationUpdate-init-validation>
        pass
        # </GSL customizable: AccelerationUpdate-init-validation>

    # <default GSL customizable: AccelerationUpdate-extra-members />

    def _serialize(self, msg: imu_pb2.ImuMessage) -> None:
        msg.kind = ACCELERATION
        msg.x = self.x
        msg.y = self.y
        msg.z = self.z
        msg.subscription.CopyFrom(self.subscription)


@protobuf.message(imu_pb2.ImuMessage, 'imu_message', fields=())
@dataclass(frozen=True, repr=False)
class PoseRequest(Message):

    def __post_init__(self):
        # <default GSL customizable: PoseRequest-init-validation>
        pass
        # </GSL customizable: PoseRequest-init-validation>

    # <default GSL customizable: PoseRequest-extra-members />

    def _serialize(self, msg: imu_pb2.ImuMessage) -> None:
        msg.kind = POSE


@protobuf.message(imu_pb2.ImuMessage, 'imu_message', fields=('x', 'y', 'z',))
@dataclass(frozen=True, repr=False)
class PoseReply(Message):
    x: int
    y: int
    z: int

    def __post_init__(self):
        # <default GSL customizable: PoseReply-init-validation>
        pass
        # </GSL customizable: PoseReply-init-validation>

    # <default GSL customizable: PoseReply-extra-members />

    def _serialize(self, msg: imu_pb2.ImuMessage) -> None:
        msg.kind = POSE
        msg.x = self.x
        msg.y = self.y
        msg.z = self.z


@protobuf.message(imu_pb2.ImuMessage, 'imu_message', fields=('subscription',))
@dataclass(frozen=True, repr=False)
class PoseSubscribe(Message):
    subscription: Subscription

    def __post_init__(self):
        # <default GSL customizable: PoseSubscribe-init-validation>
        pass
        # </GSL customizable: PoseSubscribe-init-validation>

    # <default GSL customizable: PoseSubscribe-extra-members />

    def _serialize(self, msg: imu_pb2.ImuMessage) -> None:
        msg.kind = POSE
        msg.subscription.CopyFrom(self.subscription)


@protobuf.message(imu_pb2.ImuMessage, 'imu_message', fields=('x', 'y', 'z', 'subscription',))
@dataclass(frozen=True, repr=False)
class PoseUpdate(Message):
    is_async = True

    x: int
    y: int
    z: int
    subscription: Subscription

    def __post_init__(self):
        # <default GSL customizable: PoseUpdate-init-validation>
        pass
        # </GSL customizable: PoseUpdate-init-validation>

    # <default GSL customizable: PoseUpdate-extra-members />

    def _serialize(self, msg: imu_pb2.ImuMessage) -> None:
        msg.kind = POSE
        msg.x = self.x
        msg.y = self.y
        msg.z = self.z
        msg.subscription.CopyFrom(self.subscription)


@RequestMsg.parser('imu_message')
def _parse_imu_message_request(msg: imu_pb2.ImuMessage) -> Union[RateRequest, RateSubscribe, AccelerationRequest, AccelerationSubscribe, PoseRequest, PoseSubscribe]:
    kind = msg.kind
    x = msg.x
    y = msg.y
    z = msg.z
    subscription = msg.subscription if msg.HasField('subscription') else None
    # <GSL customizable: _parse_imu_message_request-return>
    if not subscription:
        class_ = {
            RATE: RateRequest,
            ACCELERATION: AccelerationRequest,
            POSE: PoseRequest,
        }[kind]
        return class_()
    else:
        class_ = {
            RATE: RateSubscribe,
            ACCELERATION: AccelerationSubscribe,
            POSE: PoseSubscribe,
        }[kind]
        return class_(subscription)
    # </GSL customizable: _parse_imu_message_request-return>


@ReplyMsg.parser('imu_message')
def _parse_imu_message_reply(msg: imu_pb2.ImuMessage) -> Union[RateReply, RateUpdate, AccelerationReply, AccelerationUpdate, PoseReply, PoseUpdate]:
    kind = msg.kind
    x = msg.x
    y = msg.y
    z = msg.z
    subscription = msg.subscription if msg.HasField('subscription') else None
    # <GSL customizable: _parse_imu_message_reply-return>
    if not subscription:
        class_ = {
            RATE: RateReply,
            ACCELERATION: AccelerationReply,
            POSE: PoseReply,
        }[kind]
        return class_(x, y, z)
    else:
        class_ = {
            RATE: RateUpdate,
            ACCELERATION: AccelerationUpdate,
            POSE: PoseUpdate,
        }[kind]
        return class_(x, y, z, subscription)
    # </GSL customizable: _parse_imu_message_reply-return>



from typing import Any, Sequence, Union
from dataclasses import dataclass

from . import RequestMsg, ReplyMsg, Message, SimpleMessage
from hedgehog.protocol.proto import motor_pb2
from hedgehog.utils import protobuf

__all__ = ['Action', 'ConfigAction', 'CommandRequest', 'CommandReply', 'CommandSubscribe', 'CommandUpdate', 'StateRequest', 'StateReply', 'StateSubscribe', 'StateUpdate', 'SetPositionAction']

# <GSL customizable: module-header>
from hedgehog.protocol.errors import InvalidCommandError
from hedgehog.protocol.proto.motor_pb2 import POWER, BRAKE, VELOCITY
from hedgehog.protocol.proto.subscription_pb2 import Subscription


@dataclass
class DcConfig:
    pass


@dataclass
class EncoderConfig:
    encoder_a_port: int
    encoder_b_port: int


@dataclass
class StepperConfig:
    pass


Config = Union[DcConfig, EncoderConfig, StepperConfig]


__all__ += [
    'DcConfig', 'EncoderConfig', 'StepperConfig', 'Config',
    'POWER', 'BRAKE', 'VELOCITY',
    'Subscription',
]
# </GSL customizable: module-header>


@RequestMsg.message(motor_pb2.MotorAction, 'motor_action', fields=('port', 'state', 'amount', 'reached_state', 'relative', 'absolute',))
@dataclass(frozen=True, repr=False)
class Action(SimpleMessage):
    port: int
    state: int
    amount: int = 0
    reached_state: int = POWER
    relative: int = None
    absolute: int = None

    def __post_init__(self):
        # <GSL customizable: Action-init-validation>
        if self.relative is not None and self.absolute is not None:
            raise InvalidCommandError("relative and absolute are mutually exclusive")
        if self.relative is None and self.absolute is None:
            if self.reached_state != 0:
                raise InvalidCommandError(
                    "reached_state must be kept at its default value for non-positional motor commands")
        else:
            if self.state == BRAKE:
                raise InvalidCommandError("state can't be BRAKE for positional motor commands")
            if self.amount <= 0:
                raise InvalidCommandError("velocity/power must be positive for positional motor commands")
        # </GSL customizable: Action-init-validation>

    # <default GSL customizable: Action-extra-members />

    @classmethod
    def _parse(cls, msg: motor_pb2.MotorAction) -> 'Action':
        port = msg.port
        state = msg.state
        amount = msg.amount
        reached_state = msg.reached_state
        relative = msg.relative if msg.HasField('relative') else None
        absolute = msg.absolute if msg.HasField('absolute') else None
        return cls(port, state, amount, reached_state=reached_state, relative=relative, absolute=absolute)

    def _serialize(self, msg: motor_pb2.MotorAction) -> None:
        msg.port = self.port
        msg.state = self.state
        msg.amount = self.amount
        msg.reached_state = self.reached_state
        if self.relative is not None:
            msg.relative = self.relative
        if self.absolute is not None:
            msg.absolute = self.absolute


@RequestMsg.message(motor_pb2.MotorConfigAction, 'motor_config_action', fields=('port', 'config',))
@dataclass(frozen=True, repr=False)
class ConfigAction(SimpleMessage):
    port: int
    config: Any

    def __post_init__(self):
        # <default GSL customizable: ConfigAction-init-validation>
        pass
        # </GSL customizable: ConfigAction-init-validation>

    # <default GSL customizable: ConfigAction-extra-members />

    @classmethod
    def _parse(cls, msg: motor_pb2.MotorConfigAction) -> 'ConfigAction':
        port = msg.port
        # <GSL customizable: ConfigAction-parse-config>
        if msg.HasField('dc'):
            config = DcConfig()
        elif msg.HasField('encoder'):
            config = EncoderConfig(msg.encoder.encoder_a_port, msg.encoder.encoder_b_port)
        elif msg.HasField('stepper'):
            config = StepperConfig()
        else:  # pragma: nocover
            assert False
        # </GSL customizable: ConfigAction-parse-config>
        return cls(port, config)

    def _serialize(self, msg: motor_pb2.MotorConfigAction) -> None:
        msg.port = self.port
        # <GSL customizable: ConfigAction-serialize-config>
        if isinstance(self.config, DcConfig):
            msg.dc.SetInParent()
        elif isinstance(self.config, EncoderConfig):
            msg.encoder.encoder_a_port = self.config.encoder_a_port
            msg.encoder.encoder_b_port = self.config.encoder_b_port
        elif isinstance(self.config, StepperConfig):
            msg.stepper.SetInParent()
        else:  # pragma: nocover
            assert False
        # </GSL customizable: ConfigAction-serialize-config>


@protobuf.message(motor_pb2.MotorCommandMessage, 'motor_command_message', fields=('port',))
@dataclass(frozen=True, repr=False)
class CommandRequest(Message):
    port: int

    def __post_init__(self):
        # <default GSL customizable: CommandRequest-init-validation>
        pass
        # </GSL customizable: CommandRequest-init-validation>

    # <default GSL customizable: CommandRequest-extra-members />

    def _serialize(self, msg: motor_pb2.MotorCommandMessage) -> None:
        msg.port = self.port


@protobuf.message(motor_pb2.MotorCommandMessage, 'motor_command_message', fields=('port', 'config', 'state', 'amount',))
@dataclass(frozen=True, repr=False)
class CommandReply(Message):
    port: int
    config: Any
    state: int
    amount: int

    def __post_init__(self):
        # <default GSL customizable: CommandReply-init-validation>
        pass
        # </GSL customizable: CommandReply-init-validation>

    # <default GSL customizable: CommandReply-extra-members />

    def _serialize(self, msg: motor_pb2.MotorCommandMessage) -> None:
        msg.port = self.port
        # <GSL customizable: CommandReply-serialize-config>
        if isinstance(self.config, DcConfig):
            msg.dc.SetInParent()
        elif isinstance(self.config, EncoderConfig):
            msg.encoder.encoder_a_port = self.config.encoder_a_port
            msg.encoder.encoder_b_port = self.config.encoder_b_port
        elif isinstance(self.config, StepperConfig):
            msg.stepper.SetInParent()
        else:  # pragma: nocover
            assert False
        # </GSL customizable: CommandReply-serialize-config>
        msg.state = self.state
        msg.amount = self.amount


@protobuf.message(motor_pb2.MotorCommandMessage, 'motor_command_message', fields=('port', 'subscription',))
@dataclass(frozen=True, repr=False)
class CommandSubscribe(Message):
    port: int
    subscription: Subscription

    def __post_init__(self):
        # <default GSL customizable: CommandSubscribe-init-validation>
        pass
        # </GSL customizable: CommandSubscribe-init-validation>

    # <default GSL customizable: CommandSubscribe-extra-members />

    def _serialize(self, msg: motor_pb2.MotorCommandMessage) -> None:
        msg.port = self.port
        msg.subscription.CopyFrom(self.subscription)


@protobuf.message(motor_pb2.MotorCommandMessage, 'motor_command_message', fields=('port', 'config', 'state', 'amount', 'subscription',))
@dataclass(frozen=True, repr=False)
class CommandUpdate(Message):
    is_async = True

    port: int
    config: Any
    state: int
    amount: int
    subscription: Subscription

    def __post_init__(self):
        # <default GSL customizable: CommandUpdate-init-validation>
        pass
        # </GSL customizable: CommandUpdate-init-validation>

    # <default GSL customizable: CommandUpdate-extra-members />

    def _serialize(self, msg: motor_pb2.MotorCommandMessage) -> None:
        msg.port = self.port
        # <GSL customizable: CommandUpdate-serialize-config>
        if isinstance(self.config, DcConfig):
            msg.dc.SetInParent()
        elif isinstance(self.config, EncoderConfig):
            msg.encoder.encoder_a_port = self.config.encoder_a_port
            msg.encoder.encoder_b_port = self.config.encoder_b_port
        elif isinstance(self.config, StepperConfig):
            msg.stepper.SetInParent()
        else:  # pragma: nocover
            assert False
        # </GSL customizable: CommandUpdate-serialize-config>
        msg.state = self.state
        msg.amount = self.amount
        msg.subscription.CopyFrom(self.subscription)


@protobuf.message(motor_pb2.MotorStateMessage, 'motor_state_message', fields=('port',))
@dataclass(frozen=True, repr=False)
class StateRequest(Message):
    port: int

    def __post_init__(self):
        # <default GSL customizable: StateRequest-init-validation>
        pass
        # </GSL customizable: StateRequest-init-validation>

    # <default GSL customizable: StateRequest-extra-members />

    def _serialize(self, msg: motor_pb2.MotorStateMessage) -> None:
        msg.port = self.port


@protobuf.message(motor_pb2.MotorStateMessage, 'motor_state_message', fields=('port', 'velocity', 'position',))
@dataclass(frozen=True, repr=False)
class StateReply(Message):
    port: int
    velocity: int
    position: int

    def __post_init__(self):
        # <default GSL customizable: StateReply-init-validation>
        pass
        # </GSL customizable: StateReply-init-validation>

    # <default GSL customizable: StateReply-extra-members />

    def _serialize(self, msg: motor_pb2.MotorStateMessage) -> None:
        msg.port = self.port
        msg.velocity = self.velocity
        msg.position = self.position


@protobuf.message(motor_pb2.MotorStateMessage, 'motor_state_message', fields=('port', 'subscription',))
@dataclass(frozen=True, repr=False)
class StateSubscribe(Message):
    port: int
    subscription: Subscription

    def __post_init__(self):
        # <default GSL customizable: StateSubscribe-init-validation>
        pass
        # </GSL customizable: StateSubscribe-init-validation>

    # <default GSL customizable: StateSubscribe-extra-members />

    def _serialize(self, msg: motor_pb2.MotorStateMessage) -> None:
        msg.port = self.port
        msg.subscription.CopyFrom(self.subscription)


@protobuf.message(motor_pb2.MotorStateMessage, 'motor_state_message', fields=('port', 'velocity', 'position', 'subscription',))
@dataclass(frozen=True, repr=False)
class StateUpdate(Message):
    is_async = True

    port: int
    velocity: int
    position: int
    subscription: Subscription

    def __post_init__(self):
        # <default GSL customizable: StateUpdate-init-validation>
        pass
        # </GSL customizable: StateUpdate-init-validation>

    # <default GSL customizable: StateUpdate-extra-members />

    def _serialize(self, msg: motor_pb2.MotorStateMessage) -> None:
        msg.port = self.port
        msg.velocity = self.velocity
        msg.position = self.position
        msg.subscription.CopyFrom(self.subscription)


@RequestMsg.message(motor_pb2.MotorSetPositionAction, 'motor_set_position_action', fields=('port', 'position',))
@dataclass(frozen=True, repr=False)
class SetPositionAction(SimpleMessage):
    port: int
    position: int

    def __post_init__(self):
        # <default GSL customizable: SetPositionAction-init-validation>
        pass
        # </GSL customizable: SetPositionAction-init-validation>

    # <default GSL customizable: SetPositionAction-extra-members />

    @classmethod
    def _parse(cls, msg: motor_pb2.MotorSetPositionAction) -> 'SetPositionAction':
        port = msg.port
        position = msg.position
        return cls(port, position)

    def _serialize(self, msg: motor_pb2.MotorSetPositionAction) -> None:
        msg.port = self.port
        msg.position = self.position


@RequestMsg.parser('motor_command_message')
def _parse_motor_command_message_request(msg: motor_pb2.MotorCommandMessage) -> Union[CommandRequest, CommandSubscribe]:
    port = msg.port
    dc = msg.dc if msg.HasField('dc') else None
    encoder = msg.encoder if msg.HasField('encoder') else None
    stepper = msg.stepper if msg.HasField('stepper') else None
    state = msg.state
    amount = msg.amount
    subscription = msg.subscription if msg.HasField('subscription') else None
    # <GSL customizable: _parse_motor_command_message_request-return>
    if subscription is None:
        return CommandRequest(port)
    else:
        return CommandSubscribe(port, subscription)
    # </GSL customizable: _parse_motor_command_message_request-return>


@ReplyMsg.parser('motor_command_message')
def _parse_motor_command_message_reply(msg: motor_pb2.MotorCommandMessage) -> Union[CommandReply, CommandUpdate]:
    port = msg.port
    dc = msg.dc if msg.HasField('dc') else None
    encoder = msg.encoder if msg.HasField('encoder') else None
    stepper = msg.stepper if msg.HasField('stepper') else None
    state = msg.state
    amount = msg.amount
    subscription = msg.subscription if msg.HasField('subscription') else None
    # <GSL customizable: _parse_motor_command_message_reply-return>
    if dc:
        config = DcConfig()
    elif encoder:
        config = EncoderConfig(msg.encoder.encoder_a_port, msg.encoder.encoder_b_port)
    elif stepper:
        config = StepperConfig()
    else:  # pragma: nocover
        assert False

    if subscription is None:
        return CommandReply(port, config, state, amount)
    else:
        return CommandUpdate(port, config, state, amount, subscription)
    # </GSL customizable: _parse_motor_command_message_reply-return>


@RequestMsg.parser('motor_state_message')
def _parse_motor_state_message_request(msg: motor_pb2.MotorStateMessage) -> Union[StateRequest, StateSubscribe]:
    port = msg.port
    velocity = msg.velocity
    position = msg.position
    subscription = msg.subscription if msg.HasField('subscription') else None
    # <GSL customizable: _parse_motor_state_message_request-return>
    if subscription is None:
        return StateRequest(port)
    else:
        return StateSubscribe(port, subscription)
    # </GSL customizable: _parse_motor_state_message_request-return>


@ReplyMsg.parser('motor_state_message')
def _parse_motor_state_message_reply(msg: motor_pb2.MotorStateMessage) -> Union[StateReply, StateUpdate]:
    port = msg.port
    velocity = msg.velocity
    position = msg.position
    subscription = msg.subscription if msg.HasField('subscription') else None
    # <GSL customizable: _parse_motor_state_message_reply-return>
    if subscription is None:
        return StateReply(port, velocity, position)
    else:
        return StateUpdate(port, velocity, position, subscription)
    # </GSL customizable: _parse_motor_state_message_reply-return>



from hedgehog.protocol import CommSide
from hedgehog.utils.zmq.trio import Socket

from . import raw_to_delimited, to_delimited, raw_from_delimited, from_delimited
from .asyncio import DealerRouterMixin, ReqMixin

__all__ = [
    'raw_to_delimited', 'to_delimited', 'raw_from_delimited', 'from_delimited',
    'DealerRouterMixin', 'DealerRouterSocket', 'ReqMixin', 'ReqSocket',
]


class DealerRouterSocket(DealerRouterMixin, Socket):
    side = None

    def __init__(self, *args, side: CommSide, **kwargs) -> None:
        super(DealerRouterSocket, self).__init__(*args, **kwargs)
        self.side = side


class ReqSocket(ReqMixin, Socket):
    side = None

    def __init__(self, *args, side: CommSide, **kwargs) -> None:
        super(ReqSocket, self).__init__(*args, **kwargs)
        self.side = side
